/* Program to estimate Moulton standard errors with district and MSA components for Hoxby project */

#delimit;

capture program drop revarmat;
program define revarmat, rclass;
 /***
  *** This program takes as arguments a variable denoting the group to which each
  *** observation belongs and a set of variables, X.  It returns the following
  *** matrices (in r()):
  *** r(inner):    X'WX
  *** r(outer):    X'X
  *** r(invouter): inv(X'X)
  *** r(V)       : inv(X'X)*X'WX*inv(X'X),
  ***   where W is a block-diagonal weighting matrix,
  ***   w{i,j}=1 if i and j are in the same group
  ***   w{i,j}=0 otherwise.
  ***
  *** revarmat is useful, for example, in a random effects setting, where
  ***             y_ij=X_ij*b+u_j+e_ij.
  *** Here,
  ***       var(b_ols)=sigma2_e*inv(X'X) + sigma2_u*V,
  *** where V is the output from "revarmat j X"
  ***
  *** by Jesse Rothstein, April 13, 2003
  ***/

 syntax varlist (min=2) [if] [pweight aw fw iw];

 tokenize `varlist';
 local groupvar "`1'";
 macro shift 1;
 local xvars "`*'";

 marksample touse;

 *Make the inner ones matrices;
 sort `groupvar';
 tempvar ningroup matname row;
 by `groupvar': egen `ningroup'=count(`groupvar') if `touse';
 quietly tabulate `ningroup' if `touse', matrow(gpsizes);
 quietly gen str8 `matname'="";
 local i=1;
 while `i'<=rowsof(gpsizes) {;
   matrix ones`i'=J(gpsizes[`i', 1], gpsizes[`i', 1], 1);
   quietly replace `matname'="ones`i'" if `ningroup'==gpsizes[`i', 1] & `touse';
   local i=`i'+1;
   };
 quietly gen `row'=1;
 matrix glsaccum inner=`xvars' if `touse' [`weight'`exp'],
   group(`groupvar') glsmat(`matname') row(`row');
 matrix accum outer=`xvars' if `touse' [`weight'`exp'];
 matrix invouter=syminv(outer);
 matrix group=invouter*inner*invouter;
 return matrix inner inner;
 return matrix outer outer;
 return matrix invouter invouter;
 return matrix V group;
end;

/* This program estimates an OLS model with error components ("Moulton") standard errors,
   with up to three grouping levels (i.e. school, district, and MSA) */
capture program drop olserrcomp;
program define olserrcomp, eclass;
 version 7.0;
 if ~replay() {;
   syntax varlist [if] [pweight aw iw], GRoups(varlist min=1 max=3) [Level(integer $S_level)];

   /* Parse syntax */
   tokenize `varlist';
   local yvar="`1'";
   macro shift 1;
   local xvars "`*'";
   tokenize `groups';
   local i=1;
   while "``i''"~="" {;
     local gpvar`i'="``i''";
     local ngpvars=`i';
     local i=`i'+1;
   };

   marksample touse;
   preserve;
   quietly reg `varlist' if `touse' [`weight'`exp'];
   tempvar samp resids;
   gen `samp'=e(sample);
   predict `resids' if `samp'==1, residuals;
   matrix betas=e(b);
   local nxvars=e(df_m);
   local Nobs=e(N);

   if `ngpvars'==1 {;
     errcomp2 `xvars' if `touse', e(`resids') groups(`gpvar1');
     local var0=r(var0);
     local var1=r(var1);
     local ng1=r(Ngroups);
     revarmat `gpvar1' `xvars' if `touse' [`weight'`exp'];
     matrix var=`var1'*r(V) + `var0'*r(invouter);
   };
   else if `ngpvars'==2 {;
     tempvar gpcounter1;
     errcomp3 `xvars' if `touse', e(`resids') groups(`gpvar1' `gpvar2');
     local var0=r(var0);
     local var1=r(var1);
     local var2=r(var2);
     local ng1=r(Ngroups1);
     local ng2=r(Ngroups2);
     sort `gpvar1' `gpvar2';
     quietly egen `gpcounter1'=group(`gpvar1' `gpvar2') if `touse', missing;
     quietly revarmat `gpcounter1' `xvars' if `touse' [`weight'`exp'];
     matrix V1=r(V);
     quietly revarmat `gpvar1' `xvars' if `touse' [`weight'`exp'];
     matrix V2=r(V);
     matrix var=`var2'*V2 + `var1'*V1 + `var0'*r(invouter);
   };

   restore;
   estimates post betas var, esample(`touse') depname(`yvar') obs(`Nobs');
   estimates local depvar "`yvar'";
   estimates scalar df_m = `nxvars';
   estimates local model = "OLS-re`ngpvars'";
   estimates scalar sigma_0=sqrt(`var0');
   local i=1;
   while `i'<=`ngpvars' {;
     estimates scalar N_g`i' = `ng`i'';
     estimates scalar sigma_`i'=sqrt(`var`i'');
     local i=`i'+1;
   };
   estimates local vcetype "RE`ngpvars'-robust";
   estimates local groupvars "`groups'";
   estimates scalar Nlevels=`ngpvars'+1;
   estimates local cmd "olserrcomp";
 };
 else {;  /* replay */
   if "`e(cmd)'"~="olserrcomp" {;
     error 301;
   };
   syntax [, Level(integer $S_level)];
 };
 if `level'<10 | `level'>99 {;
   di as err "level() must be between 10 and 99, inclusive";
   exit 198;
 };
 /* Display results */
 di _n in gr "OLS regression, with error-component-robust variance" _n;
 di   _col(49) in gr "Number of obs" _col(68) "="
	_col(70) in ye %9.0f e(N) ;
 di   _col(49) in gr "Number of levels" _col(68) "="
	_col(70) in ye %9.0f e(Nlevels) _n;
 estimates display, plus level(`level');
 di in smcl in gr "Level var.   {c |}    S.D.      # of groups ";
 tokenize `e(groupvars)';
 local i=1;
 while `i'<e(Nlevels) {;
   local j=e(Nlevels)-`i';
   di in smcl in gr "  ``i''" _col(14) "{c |} " in ye %10.0g e(sigma_`j') %10.0g e(N_g`j');
   local i=`i'+1;
 };
 di in smcl in gr "  residual   {c |} " in ye %10.0g e(sigma_0) "        --";
 di in smcl in gr "{hline 13}{c BT}{hline 64}";
end;

/* This program estimates an IV model with error components ("Moulton") standard errors,
   with up to three grouping levels (i.e. school, district, and MSA) */
capture program drop iverrcomp;
program define iverrcomp, eclass;
 version 7.0;
 if ~replay() {;
   gettoken varlistA 0: 0, parse("(");
   gettoken varlistB 0: 0, match(testparn);
   if `"`testparn'"'~="(" {;
     di in red `"Syntax is "yvar xvars (endogx=instruments)""';
     exit 198;
   };
   gettoken endog instrum: varlistB, parse("=");
   gettoken eqsign instrum: instrum, parse("=");
   capture confirm numeric variable `varlistA';
   if _rc==0 {;
     tsunab varlistA : `varlistA', name(Varlist);
   };
   else {;
     di in red `"Syntax is "yvar xvars (endogx=instruments)", all numeric"';
     exit 198;
   };
   capture confirm numeric variable `endog';
   if _rc==0 {;
     tsunab endog    : `endog',    name(Endogvars);
   };
   else {;
     di in red `"Syntax is "yvar xvars (endogx=instruments)", all numeric"';
     exit 198;
   };
   capture confirm numeric variable `instrum';
   if _rc==0 {;
     tsunab instrum  : `instrum',  name(Instrumvars);
   };
   else {;
     di in red `"Syntax is "yvar xvars (endogx=instruments)", all numeric"';
     exit 198;
   };
   syntax [if] [pweight aw fw], GRoups(varlist min=1 max=3) [Level(integer $S_level)];
   tokenize `varlistA';
   local yvar=`"`1'"';
   macro shift 1;
   local exog `"`*'"';
   tokenize `exog';
   local newexog="";
   local numxvars=0;
   local i=1;
   while "``i''"~="" {;
     if index(" `newexog' ", " ``i'' ")==0 {;
       local newexog `"`newexog' ``i''"';
       local numxvars=`numxvars'+1;
     };
     local i=`i'+1;
   };

   tokenize `endog';
   local newendog="";
   local numnvars=0;
   local i=1;
   while "``i''"~="" {;
     if index(" `newendog' ", " ``i'' ")==0 & index(" `newexog' ", " ``i'' ")==0 {;
       local newendog `"`newendog' ``i''"';
       local numnvars=`numnvars'+1;
     };
     local i=`i'+1;
   };
   if `numnvars'==0 {;
     di in red "Must specify at least one endogenous variable";
     exit 481;
   };

   tokenize `instrum';
   local newinst="";
   local numzvars=0;
   local i=1;
   while "``i''"~="" {;
     if index(" `newinst' ", " ``i'' ")==0 & index(" `newexog' ", " ``i'' ")==0 &
        index(" `newendog' ", " ``i'' ")==0 {;
       local newinst `"`newinst' ``i''"';
       local numzvars=`numzvars'+1;
     };
     local i=`i'+1;
   };
   if `numnvars' > `numzvars' {;
     di in red "equation not identified; must have at least as many instruments not in";
     di in red "the regression as there are instrumented variables";
     exit 481;
   };

   if `numxvars'>0 {;
     tsunab exog : `newexog';
   };
   if `numnvars'>0 {;
     tsunab endog : `newendog';
   };
   if `numzvars'>0 {;
     tsunab instrum : `newinst';
   };
   tokenize `groups';
   local i=1;
   while "``i''"~="" {;
     local gpvar`i'="``i''";
     local numgpvars=`i';
     local i=`i'+1;
   };

   /* Done parsing syntax */
   marksample touse;
   markout `touse' `yvar' `exog' `endog' `instrum' `groups';
   tempvar ivresid xhat;

   /* Run basic regression, obtain coefficients and residuals */
   quietly ivreg `yvar' `exog' (`endog'=`instrum') if `touse' [`weight'`exp'];
   matrix betaiv=e(b);
   local Nobs=e(N);
   quietly predict `ivresid' if `touse', r;
   *Also do first stage regression, to get xhat;
   quietly reg `yvar' `exog' `instrum' if `touse' [`weight'`exp'];
   predict `xhat' if e(sample), xb;

   /* Create first-level matrices, read off design matrices */
   tempvar gpcounter1;
   sort `groups';
   quietly egen `gpcounter1'=group(`groups') if `touse', missing;
   quietly revarmat `gpcounter1' `exog' `instrum' if `touse' [`weight'`exp'];
   matrix V1=r(inner);
   quietly matrix accum XZXZ=`endog' `exog' `instrum' if `touse';

   matrix WW=XZXZ[1..`numnvars', 1..`numnvars'];
   matrix WZ=XZXZ[1..`numnvars',
                  `numnvars'+`numxvars'+1..`numnvars'+`numxvars'+`numzvars'];
   matrix W1=XZXZ[1..`numnvars',
                  `numnvars'+`numxvars'+`numzvars'+1];
   matrix ZZ=XZXZ[`numnvars'+`numxvars'+1..`numnvars'+`numxvars'+`numzvars',
                  `numnvars'+`numxvars'+1..`numnvars'+`numxvars'+`numzvars'];
   matrix Z1=XZXZ[`numnvars'+`numxvars'+1..`numnvars'+`numxvars'+`numzvars',
                  `numnvars'+`numxvars'+`numzvars'+1];
   matrix oneone=XZXZ[`numnvars'+`numxvars'+`numzvars'+1, `numnvars'+`numxvars'+`numzvars'+1];
   if `numxvars'>0 {;
     matrix XX=XZXZ[`numnvars'+1..`numnvars'+`numxvars',
                    `numnvars'+1..`numnvars'+`numxvars'];
     matrix XZ=XZXZ[`numnvars'+1..`numnvars'+`numxvars',
                    `numnvars'+`numxvars'+1..`numnvars'+`numxvars'+`numzvars'];
     matrix X1=XZXZ[`numnvars'+1..`numnvars'+`numxvars',
                    `numnvars'+`numxvars'+`numzvars'+1];
     matrix WX=XZXZ[1..`numnvars',
                    `numnvars'+1..`numnvars'+`numxvars'];
     matrix X1X1=(WW, WX, W1) \ (WX', XX, X1) \ (W1', X1', oneone);
     matrix X1Z1=(WX, WZ, W1) \ (XX, XZ, X1) \ (X1', Z1', oneone);
     matrix Z1Z1=(XX, XZ, X1) \ (XZ', ZZ, Z1) \ (X1', Z1', oneone);
   };
   else {;
     matrix X1X1=(WW, W1) \ (W1', oneone);
     matrix X1Z1=(WZ, W1) \ (Z1', oneone);
     matrix Z1Z1=(ZZ, Z1) \ (Z1', oneone);
   };
   matrix outeriv=syminv(X1Z1 * syminv(Z1Z1) * X1Z1')*X1Z1*syminv(Z1Z1);

   /* Estimate variance components and variance matrix, based on number of error components */
   if `numgpvars'==1 {;
     quietly errcomp2 `exog' `xhat' if `touse', e(`ivresid') groups(`gpvar1');
     local var0=r(var0);
     local var1=r(var1);
     local ng1=r(Ngroups);
     matrix inner=`var1'*V1 + `var0'*Z1Z1;
   };
   else if `numgpvars'==2 {;
     quietly errcomp3 `exog' `xhat' if `touse', e(`ivresid') groups(`gpvar1' `gpvar2');
     local var0=r(var0);
     local var1=r(var1);
     local var2=r(var2);
     local ng1=r(Ngroups1);
     local ng2=r(Ngroups2);
     sort `gpvar1' `gpvar2';
     quietly revarmat `gpvar1' `exog' `instrum' if `touse' [`weight'`exp'];
     matrix V2=r(inner);
     matrix inner=`var2'*V2+`var1'*V1 + `var0'*Z1Z1;
   };
   matrix var=outeriv*inner*outeriv';

   estimates post betaiv var, esample(`touse') depname(`yvar') obs(`Nobs');
   estimates local depvar "`yvar'";
   estimates scalar df_m = `numxvars'+`numnvars';
   estimates scalar N_exog = `numxvars';
   estimates local model = "IV-re`numgpvars'";
   estimates scalar sigma_0=sqrt(`var0');
   estimates local wtype="`weight'";
   estimates local wexp "`exp'";
   estimates local instd `endog';
   estimates local insts `instrum';
   estimates local exogx `exog';
   local i=1;
   while `i'<=`numgpvars' {;
     estimates scalar N_g`i' = `ng`i'';
     estimates scalar sigma_`i'=sqrt(`var`i'');
     local i=`i'+1;
   };
   estimates local vcetype "RE`ngpvars'-robust";
   estimates local groupvars "`groups'";
   estimates scalar Nlevels=`numgpvars'+1;
   estimates local cmd "iverrcomp";
 };
 else {;  /* replay */
   if "`e(cmd)'"~="iverrcomp" {;
     error 301;
   };
   syntax [, Level(integer $S_level)];
 };
 if `level'<10 | `level'>99 {;
   di as err "level() must be between 10 and 99, inclusive";
   exit 198;
 };
 /* Display results */
 di _n in gr "IV regression, with error-component-robust variance" _n;
 di   _col(49) in gr "Number of obs" _col(68) "="
	_col(70) in ye %9.0f e(N) ;
 di   _col(49) in gr "Number of levels" _col(68) "="
	_col(70) in ye %9.0f e(Nlevels) _n;
 estimates display, plus level(`level');
 di in smcl in gr "Level var.   {c |}    S.D.      # of groups ";
 tokenize `e(groupvars)';
 local i=1;
 while `i'<e(Nlevels) {;
   local j=e(Nlevels)-`i';
   di in smcl in gr "  ``i''" _col(14) "{c |} " in ye %10.0g e(sigma_`j') %10.0g e(N_g`j');
   local i=`i'+1;
 };
 di in smcl in gr "  residual   {c |} " in ye %10.0g e(sigma_0) "        --";
 di in smcl in gr "{hline 13}{c BT}{hline 64}";
 if e(N_exog)>0 {;
   di in gr "Exogenous:     " _c;
   Disp `e(exogx)';
 };
 di in gr "Instrumented:  " _c;
 Disp `e(instd)';
 di in gr "Instruments:   " _c;
 Disp `e(insts)';
 di in smcl in gr "{hline 78}";
end;

capture program drop Disp;
program define Disp ;
	local first "";
	local piece : piece 1 64 of `"`0'"';
	local i 1;
	while "`piece'" != "" {;
		di in gr "`first'`piece'";
		local first "               ";
		local i = `i' + 1;
		local piece : piece `i' 64 of `"`0'"';
	};
	if `i'==1 { di };
end;

/* This program computes the variance of the error components when there is just one group variable
   (i.e. two error components, individual and group) */
capture program drop errcomp2;
program define errcomp2, rclass;
 * Compute error components with just one group variable.;
 version 7.0;
 if ~replay() {;
   syntax varlist [if] , e(varname) GRoups(varname) [Level(integer $S_level)];
   *Note:  Procedure doesnt take weights.
   *       With pweights, it shouldnt matter unless there is heteroskedasticity
   *        (in which case the model is misspecified).  Aweights assume heteroskedasticity.;

   marksample touse;
   preserve;
   quietly keep if `touse';

   *Calculate error components.  e_ij=u_j+eps_ij.;
   * var(e_ij | j)=var(eps);
   * var(ebar_j)=var(u)+ (1/N_j)var(eps), so var(u)=var(ebar_j) - [avg. of (1/N_j)]*var(eps);

   * Calculate var(eps) from within-group residual.  Many controls may drop out if they do not vary within groups;
   *Run regression to compute degrees of freedom;
   quietly areg `e' `varlist' if `touse', absorb(`groups');
   local kwithin=e(df_m);
   *Now run regression without Xs to get within-group variance;
   quietly areg `e' if `touse', absorb(`groups');
   local var_eps=e(rss)/(e(N)-`kwithin'-e(df_a)-1);
   local N=e(N);

   * Calculate var(ebar_j);
       *Collapse to group level;
       sort `groups';
       collapse (mean) `e' `varlist' (count) obsingp=`e' if `touse', by(`groups');

       *Compute var(ebar_j);
          *Regress out varlist--many may drop out if they do not vary across groups;
          quietly reg `e' `varlist';
          local kbetween=e(df_m);
          su `e', meanonly;
          gen e2=(`e'-r(mean))^2;
          quietly su `e2', meanonly;
          local var_ebar=r(sum)/(r(N)-`kbetween'-1);
   * Calculate var(u);
   gen invnobs=1/obsingp;
   su invnobs, meanonly;
   local Q1=r(mean);
   local N1=r(N);
   local var_mu=max(`var_ebar' - `Q1'*`var_eps', 0);
 };

  return scalar var0=`var_eps';
  return scalar var1=`var_mu';
  return scalar N=`N';
  return scalar Ngroups=`N1';
  return local groupvar=`groups';
  return scalar Tbar=`Q1';
end;

/* This program computes the variance of the error components when there are two group variables
   (i.e. three error components, individual, subgroup, and group) */
capture program drop errcomp3;
program define errcomp3, rclass;
 * Compute error components with two group variables.;
 version 7.0;
 if ~replay() {;
   syntax varlist [if] , e(varname) GRoups(varlist min=2 max=2);
   *Note:  Procedure doesnt take weights.
   *       With pweights, it shouldnt matter unless there is heteroskedasticity
   *        (in which case the model is misspecified).  Aweights assume heteroskedasticity.;

   * Read in Group variables;
   tokenize `groups';
   local gpvar1="`1'";
   local gpvar2="`2'";

   marksample touse;
   preserve;
   quietly keep if `touse';

   sort `groups';
   egen gpcounter=group(`groups');

   *Calculate error components.  e_ijk=theta_k + u_jk + eps_ijk.;
   * var(e_ijk | j, k)=var(eps);
   * var(ebar_jk | k)=var(u)+ (1/N_jk)var(eps), so var(u)=var(ebar_jk | k) - [avg. of (1/N_jk)]*var(eps);
   * var(ebar_k) = var(theta) + (1/P_k)var(u) + (1/N_k)var(eps),
   *    so var(theta)=var(ebar_k) - [avg of (1/P_k)]*var(u) - [avg of (1/N_k)]*var(eps);

   * Calculate var(eps) from within-group residual.  Many controls may drop out if they do not vary within groups;
   *Run regression to compute degrees of freedom;
   quietly areg `e' `varlist' if `touse', absorb(gpcounter);
   local kwithin=e(df_m);
   *Now run regression without Xs to get within-group variance;
   quietly areg `e' if `touse', absorb(gpcounter);
   local var_eps=e(rss)/(e(N)-`kwithin'-e(df_a)-1);
   local N=e(N);
   *Compute var(u);
   		  *Start by getting var(ebar_jk);
   		        *Collapse to subgroup level;
   		        sort `groups';
   		        collapse (mean) `e' `varlist' (count) obsingp=`e' if `touse', by(`groups');
   		        *Compute subgroup-level variance;
   		        quietly areg `e' `varlist', absorb(`gpvar1');
   		        local kbetween=e(df_m);
   		        quietly areg `e', absorb(`gpvar1');
   		        local var_subgp=e(rss)/(e(N)-`kbetween'-e(df_a)-1);
   		  *Compute var(u);
        gen invnobs=1/obsingp;
        summarize invnobs, meanonly;
        local Q1=r(mean);
        local N1=r(N);
        local var_mu=max(`var_subgp'-`Q1'*`var_eps', 0);
   *And finally var(theta);
   		  *Start by getting var(ebar_k);
   		        *Collapse to subgroup level;
   		        sort `gpvar1';
   		        egen subsingp=tag(`gpvar1' `gpvar2');
   		        collapse (mean) `e' `varlist' (rawsum) obsingp subsingp
   		                 [fw=obsingp], by(`gpvar1');
   		        *Compute group-level variance;
   		        quietly reg `e' `varlist';
   		        local kbetweenk=e(df_m);
              su `e', meanonly;
              gen e2=(`e'-r(mean))^2;
              quietly su `e2', meanonly;
              local var_gp=r(sum)/(r(N)-`kbetweenk'-1);
   		  *Compute var(theta);
        gen invnobs=1/obsingp;
        gen invnsubs=1/subsingp;
        su invnsubs, meanonly;
        local P2=r(mean);
        su invnobs, meanonly;
        local Q2=r(mean);
        local N2=r(N);
        local var_theta=max(`var_gp'-`P2'*`var_mu'-`Q2'*`var_eps', 0);
 };

  return scalar var0=`var_eps';
  return scalar var1=`var_mu';
  return scalar var2=`var_theta';
  return scalar N=`N';
  return scalar Ngroups1=`N1';
  return scalar Ngroups2=`N2';
  return local  groupvars="`groups'";
  return scalar Tbar1=`Q1';
  return scalar Tbar2=`Q2';
  return scalar Sbar2=`P2';
end;

/* This program computes the variance of the error components when there are three group variables
   (i.e. four error components, individual, subsubgroup, subgroup, and group)
NOTE:  THIS 3-LEVEL ERROR COMPONENTS MODEL IS NO LONGER USED, AND IS NOT COMPLETELY DEBUGGED.
capture program drop errcomp4;
program define errcomp4, rclass;
 * Compute error components with three group variables.;
 version 7.0;
 if ~replay() {;
   syntax varlist [if] , e(varname) GRoups(varlist min=3 max=3);
   *Note:  Procedure doesnt take weights.
   *       With pweights, it shouldnt matter unless there is heteroskedasticity
   *        (in which case the model is misspecified).  Aweights assume heteroskedasticity.;

   * Read in Group variables;
   tokenize `groups';
   local gpvar1="`1'";
   local gpvar2="`2'";
   local gpvar3="`3'";

   marksample touse;
   preserve;
   quietly keep if `touse';

   sort `groups';
   egen subsubgpcounter=group(`groups');
   egen subgpcounter=group(`gpvar1' `gpvar2');
   egen gpcounter=group(`gpvar1');

   *Calculate error components.  e_ijkl=omega_l + theta_kl + u_jkl + eps_ijkl.;
   * var(e_ijkl | j, k, l)=var(eps);
   * var(ebar_jkl | k, l)=var(u)+ (1/N_jkl)var(eps), so var(u)=var(ebar_jkl | k, l) - [avg. of (1/N_jkl)]*var(eps);
   * var(ebar_kl | l) = var(theta) + (1/P_kl)var(u) + (1/N_kl)var(eps),
   *    so var(theta)=var(ebar_kl | l) - [avg of (1/P_kl)]*var(u) - [avg of (1/N_kl)]*var(eps);
   * var(ebar_l) = var(omega) + (1/T_l)var(theta) + (1/P_l)var(u) + (1/N_l)var(eps), so
   *    var(omega)=var(ebar_l) - [avg of (1/T_l)]var(theta) - [avg of (1/P_l)]var(u) - [avg of (1/N_l)]var(eps)

   * Calculate var(eps) from within-group residual.  Many controls may drop out if they do not vary within groups;
   *Run regression to compute degrees of freedom;
   quietly areg `e' `varlist' if `touse', absorb(`subsubgpcounter');
   local kwithin=e(df_m);
   *Now run regression without Xs to get within-group variance;
   quietly areg `e' if `touse', absorb(`gpcounter');
   local var_eps=e(rss)/(e(N)-`kwithin'-e(df_a)-1);
   local N=e(N);

   *Compute var(u);
   		  *Start by getting var(ebar_jkl);
   		        *Collapse to subgroup level;
   		        sort `groups';
   		        collapse (mean) `e' `varlist' subgpcounter gpcounter
   		                 (count) obsinsubsubgp=(`e') if `touse', by(`gpvar1' `gpvar2' `gpvar3');
   		        *Compute subgroup-level variance;
   		        quietly areg `e' `varlist' if `touse', absorb(`subgpcounter');
   		        local kbetween=e(df_m);
   		        quietly areg `e' if `touse', absorb(`subgpcounter');
   		        local var_subsubgp=e(rss)/(e(N)-`kbetween'-e(df_a)-1);
   		  *Compute var(u);
        gen invnobs=1/obsinsubsubgp;
        summarize invnobs, meanonly;
        local Q1=r(mean);
        local N1=r(N);
        local var_mu=max(`var_subgp'-`Q1'*`var_eps', 0);

   *Compute var(theta);
   		  *Start by getting var(ebar_kl);
   		        *Collapse to subgroup level;
   		        sort `gpvar1' `gpvar2';
   		        collapse (mean) `e' `varlist' (rawsum) obsinsubgp=obsinsubsubgp (count) subsubsinsubgp=`e' if `touse'
   		                 [fw=obsinsubsubgp], by(`gpvar1' `gpvar2');
   		        *Compute subgroup-level variance;
   		        quietly reg `e' `varlist' if `touse';
   		        local kbetweenkl=e(df_m);
              su `e', meanonly;
              gen e2=(`e'-r(mean))^2;
              quietly su `e2', meanonly
              local var_gp=r(sum)/(r(N)-`kbetweenkl'-1);
   		  *Compute var(theta);
        gen invnobs=1/obsinsubgp;
        gen invnsubs=1/subsubsinsubgp;
        su invnsubs, meanonly;
        local P2=r(mean);
        su invnobs, meanonly;
        local Q2=r(mean);
        local N2=r(N);
        local var_theta=max(`var_gp'-`P2'*`var_mu'-`Q2'*`var_eps', 0);

   *Compute var(omega);
   		  *Start by getting var(ebar_l);
   		        *Collapse to group level;
   		        sort `gpvar1';
   		        collapse (mean) `e' `varlist'
   		                 (rawsum) obsingp=obsinsubgp subsubsingp=subsubsinsubgp
   		                 (count) subsingp=`e' if `touse'
   		                 [fw=obsinsubgp], by(`gpvar1');
   		        *Compute group-level variance;
   		        quietly reg `e' `varlist' if `touse';
   		        local kbetweenl=e(df_m);
              su `e', meanonly;
              gen e2=(`e'-r(mean))^2;
              quietly su `e2', meanonly
              local var_gp=r(sum)/(r(N)-`kbetweenl'-1);
   		  *Compute var(omega);
        gen invnobs=1/obsingp;
        gen invnsubs=1/subsingp;
        gen invnsubsubs=1/subsubsingp;
        su invnsubsubs, meanonly;
        local P3=r(mean);
        su invnsubs, meanonly;
        local T3=r(mean);
        su invnobs, meanonly;
        local Q3=r(mean);
        local N3=r(N);
        local var_omega=max(`var_gp'-`T3'*`var_theta' - `P3'*`var_mu'-`Q3'*`var_eps', 0);
 };

  return scalar var0=`var_eps';
  return scalar var1=`var_mu';
  return scalar var2=`var_theta';
  return scalar var3=`var_omega';
  return scalar N=`N';
  return scalar Ngroups1=`N1';
  return scalar Ngroups2=`N2';
  return scalar Ngroups3=`N3';
  return local  groupvars="`groups'";
  return scalar Tbar1=`Q1';
  return scalar Tbar2=`Q2';
  return scalar Tbar3=`Q3';
  return scalar Sbar2=`P2';
  return scalar Sbar3=`P3';
  return scalar Rbar3=`T3';
end;
*/