capture log close
log using makedata_msadist.smcl, replace

/*
Program to assemble MSA and district-level data for CH paper.
At each level, define variables according to two algorithms:
 1) That followed by Hoxby in creating the Hoxby/NCES data.
    These variables are prefixed "ch_", and are referred to
    as the "close replication" covariates.;
 2) The so-called "preferred" algorithm.  These variables are
    prefixed "jr_".  See appendix A for details.
 In some cases, the two algorithms were identical.  Generally,
 these variables are given no prefix.

Note:  Data sources are not always those which Hoxby uses.  For
example, a corrected CCD crosswalk is used to assign MSAs (and is
assembled here), and the Census STF files are used in place of the
SDDB for most MSA covariates.  See Appendix A for details.  Where
there was not a good reason to deviate from Hoxby's procedures,
however, data sets from the Hoxby/NCES CD were used.
*/

#delimit;
clear;
set mem 600m;
do ../definedirs.do;
version 8.0;
pause off;
set more off;
query;
creturn list;

**Section A:  Get MSA names;
di in red "**Section A:  Get MSA names";
 *Read 1990 MSA defs.  Note--lots of invalid lines--skip these;
  quietly infix msa 1-4
        pmsa 9-12
        cmsa 17-18
        stfips 25-26
        cofips 27-29
        central 33
        townfips 41-45
        str name 49-115 using "${mydata}/MSA defs/msadefs90.txt", clear;
  *Fix an error in the text file--MSA code is left off of name line for one MSA;
   replace msa=1400 if name=="Champaign-Urbana-Rantoul, IL MSA";
  *Fix one town FIPS code that is wrong on MSA definitions file;
   replace townfips=29405 if stfips==25 & cofips==9 & townfips==25625 & name=="Haverhill city";
  drop if msa==.;
  gen fipsmsa=msa if cmsa==.;
  replace fipsmsa=pmsa if cmsa~=.;
  tempfile allmsa msanames;
  save `allmsa';
  keep if stfips==. & fipsmsa~=.;
  drop if index(name, "PR")>0;
  sort fipsmsa msa cmsa pmsa name;
  duplicates report fipsmsa msa cmsa pmsa;
  rename name msaname;
  compress;
  keep fipsmsa msaname;
  save `msanames';

**Section B:  Read STF files, and construct MSA-level covariates;
di in red "**Section B:  Read STF files, and construct MSA-level covariates";
 *Read 1990 STF-3A file (county records and town records in NE);
  *Note:  I have previously extracted records from the full STF-3A file based on their sumlev codes:;
  *gzip -dc da9782.*_data_lrec.gz |gawk 'substr($0,11,3)=="050" {print $0}' > stf3a.sumlev050.dat;
  *gzip -dc da9782.*_data_lrec.gz |gawk 'substr($0,11,3)=="060" {print $0}' > stf3a.sumlev060.dat;
  *NOTE:  check.stf3a.msacodes.do checks the MSA codes on these files against those from the official defs.
  *       Things seem fine--one inconsistent town code (with right MSA), one missing very small town.;
  tempfile stf3a_cnty stf3a_town stf3a_netown;
  *50 is county-level records;
   drop _all;
   infile using makedata.include.stf3a.dct, using(${mydata}/stf3a/stf3a.sumlev050.dat);
   gen msacmsa=real(msacmsatxt);
   gen pmsa=real(pmsatxt);
   gen fipsmsa=pmsa if pmsa<9999 & pmsa>0;
   replace fipsmsa=msacmsa if fipsmsa==. & msacmsa>0 & msacmsa<9999;
   rename statefp stfips;
   rename cnty cofips;
   rename cousubfp townfips;
   save `stf3a_cnty';
  *60 is county subdivision (i.e. town)-level records;
   drop _all;
   infile using makedata.include.stf3a.dct, using(${mydata}/stf3a/stf3a.sumlev060.dat);
   gen msacmsa=real(msacmsatxt);
   gen pmsa=real(pmsatxt);
   gen fipsmsa=pmsa if pmsa<9999 & pmsa>0;
   replace fipsmsa=msacmsa if fipsmsa==. & msacmsa>0 & msacmsa<9999;
   rename statefp stfips;
   rename cnty cofips;
   rename cousubfp townfips;
   save `stf3a_town';
   keep if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50;
   save `stf3a_netown';

  *Make County-MSA correspondence.;
   tempfile co2msa_nonne co2msa_ne co2msa;
   *Non-NE is easy;
    use stfips cofips fipsmsa using `stf3a_cnty';
    drop if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50;
    replace fipsmsa=0 if fipsmsa==.;
    sort stfips cofips;
    save `co2msa_nonne';
   *NE is a bit harder--need to use population to construct weights, and to assign largest MSA in county;
    use stfips cofips fipsmsa P1_1
     if (stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50)
     using `stf3a_town';
    rename P1_1 pop_n;
    replace fipsmsa=0 if fipsmsa==.;
    collapse (sum) pop_n, by(stfips cofips fipsmsa);
    egen copop=sum(pop_n), by(stfips cofips);
    gen ma_copopshare=pop_n/copop;
    gsort stfips cofips -ma_copopshare;
    by stfips cofips: gen ch_assocmsa=1 if _n==1 & fipsmsa>0 & fipsmsa<.;
    replace ch_assocmsa=fipsmsa if ch_assocmsa==1;
    replace ch_assocmsa=0 if ch_assocmsa==.;
    keep stfips cofips ma_copopshare ch_assocmsa fipsmsa;
    sort stfips cofips;
    save `co2msa_ne';
   *Bring together;
    use `co2msa_nonne';
    gen ma_copopshare=1;
    gen ch_assocmsa=fipsmsa;
    gen newengland=0;
    append using `co2msa_ne';
    replace newengland=1 if newengland==.;
    sort stfips cofips;
    replace fipsmsa=. if fipsmsa==0;
    replace ch_assocmsa=. if ch_assocmsa==0;
    label var fipsmsa "MSA/PMSA code (potentially one of many in NE)";
    label var ch_assocmsa "Unique MSA/PMSA, CH algorithm (plurality of cnty)";
    label var ma_copopshare "Fr. of county pop in this MSA";
    save `co2msa';

 *Separate the data into NE and non-NE;
  tempfile stf3a_nonne stf3a_ne stf3a_units;
  use `stf3a_cnty';
  drop if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50;
  drop if fipsmsa==.;
  save `stf3a_nonne';
  use `stf3a_netown';
  drop if fipsmsa==.;
  save `stf3a_ne';
  append using `stf3a_nonne';
  sort fipsmsa;
  save `stf3a_units';

 *Create variables needed to compute MSA-level variables;
  *Variables for area, population;
   rename P1_1 pop_n;
   *arealand is in square KM--change to miles;
    replace arealand=arealand/(1.609344^2);
  *Variables for racial composition;
   rename P12_11 pop_nwh;
   rename P12_12 pop_nbl;
   rename P12_13 pop_nin;
   rename P12_14 pop_nas;
   rename P12_15 pop_not;
   rename P10_1 pop_nhi;
  *Variables for age composition--not needed anymore;
   egen n_lt19=rsum(P13_1-P13_14);
   egen n_gt65=rsum(P13_27-P13_31);
  *Variables for education composition--use age 25+ (alternative is 18+, SDDB uses 20+);
   gen ed_lths=P57_1+P57_2;
   gen ed_hs=P57_3;
   gen ed_scol=P57_4+P57_5;
   gen ed_ba=P57_6+P57_7;
  *Variables for income distribution;
   *P80_1-P80_25 P81_1 P81_2;
  *Variables for ethnic distribution;
   rename P11_1 anch_nothisp;
   rename P11_2 anch_mex;
   rename P11_3 anch_pr;
   rename P11_4 anch_cubn;
   rename P11_5 anch_domn;
   rename P11_6 anch_guat;
   rename P11_7 anch_hond;
   rename P11_8 anch_nica;
   rename P11_9 anch_pana;
   rename P11_10 anch_salv;
   rename P11_11 anch_otca;
   rename P11_12 anch_colo;
   rename P11_13 anch_ecua;
   rename P11_14 anch_peru;
   rename P11_15 anch_otsa;
   rename P11_16 anch_oth;
  *Ancestry variables--non-Hispanics;
   rename P33_1 anc_arab;
   rename P33_2 anc_aust;
   rename P33_3 anc_belg;
   rename P33_4 anc_canad;
   rename P33_5 anc_czech;
   rename P33_6 anc_dansh;
   rename P33_7 anc_dutch;
   rename P33_8 anc_engl;
   rename P33_9 anc_finn;
   rename P33_10 anc_frnch;
   rename P33_11 anc_frcan;
   rename P33_12 anc_germ;
   rename P33_13 anc_grk;
   rename P33_14 anc_hung;
   rename P33_15 anc_eir;
   rename P33_16 anc_ital;
   rename P33_17 anc_lith;
   rename P33_18 anc_norw;
   rename P33_19 anc_pole;
   rename P33_20 anc_port;
   rename P33_21 anc_roman;
   rename P33_22 anc_russ;
   rename P33_23 anc_sceir;
   rename P33_24 anc_scott;
   rename P33_25 anc_slov;
   rename P33_26 anc_afric;
   rename P33_27 anc_swed;
   rename P33_28 anc_swiss;
   rename P33_29 anc_ukr;
   rename P33_30 anc_usa;
   rename P33_31 anc_welsh;
   rename P33_32 anc_windn;
   rename P33_33 anc_yugo;
   rename P33_34 anc_race;
   rename P33_35 anc_otr;
   rename P33_36 anc_uncl;
  *Compute census division;
   recode stfips  ( 9 23 25 33 44 50          =1)
                  (36 34 42                   =2)
                  (17 18 26 39 55             =3)
                  (19 20 27 29 31 38 46       =4)
                  (10 11 12 13 24 37 45 51 54 =5)
                  ( 1 21 28 47                =6)
                  ( 5 22 40 48                =7)
                  ( 4  8 16 30 32 35 49 56    =8)
                  ( 2  6 15 41 53             =9),
          gen(division);
   quietly tab division, gen(divis);
  *Variables for MSA private enrollment share;
   rename P54_3 n_pub;
   rename P54_4 n_priv;

 *Collapse to MSA level.  Do three times--once to sum everything but division,
  *once to assign division for largest state (population, where CH uses enrollment),
  *once to make variables for fraction of pop in MSA in each division;
  tempfile basedat malev1 malev2 malev3;
  sort fipsmsa stfips;
  *First just sum all but division;
   drop if fipsmsa==. | fipsmsa<=0;
   save `basedat';
   collapse (sum) arealand pop_n* n_lt19 n_gt65 ed_* P80_* P81_* anch_* anc_* n_pub n_priv, by(fipsmsa);
   sort fipsmsa;
   rename arealand m_arealand;
   rename pop_n m_pop_n;
   save `malev1';
  *Now get fraction of pop in each MSA in each division;
   use `basedat';
   collapse (mean) divis1-divis9 [fw=pop_n], by(fipsmsa);
   sort fipsmsa;
   forvalues i=1/9 {;
    rename divis`i' jr_m_divis`i';
    label var jr_m_divis`i' "Frac of MSA pop in divis `i'";
    };
   save `malev2';
  *Now assign division of largest state in MSA;
   use `basedat';
   sort fipsmsa stfips;
   collapse (sum) pop_n (mean) divis1-divis9, by(fipsmsa fipsmsa);
   gsort fipsmsa -pop_n;
   by fipsmsa: keep if _n==1;
   keep fipsmsa divis1-divis9;
   sort fipsmsa;
   forvalues i=1/9 {;
    rename divis`i' ch_m_divis`i';
    label var ch_m_divis`i' "Plurality of MSA pop is in divis `i'";
    };
   save `malev3';
  *Merge together;
   use `malev1';
   sort fipsmsa;
   merge fipsmsa using `malev2', unique _merge(haveallA);
   sort fipsmsa;
   merge fipsmsa using `malev3', unique _merge(haveallB);
   tab haveallA haveallB, missing;
   drop haveallA haveallB;

 *Start computing;
  *Create race and racial heterogeneity indices;
   *CH algorithm, ignoring other race;
    gen ch_m_pop_fwh=pop_nwh/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
    gen ch_m_pop_fbl=pop_nbl/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
    gen ch_m_pop_fin=pop_nin/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
    gen ch_m_pop_fas=pop_nas/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
    gen ch_m_pop_fhi=pop_nhi/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
    gen ch_m_herfrace=(ch_m_pop_fwh^2 + ch_m_pop_fbl^2 + ch_m_pop_fin^2 + ch_m_pop_fas^2 + ch_m_pop_fhi^2);
   *JR algorithm, including other race;
    egen jr_totrace=rsum(pop_nwh pop_nbl pop_nin pop_nas pop_nhi pop_not);
    gen jr_m_pop_fwh=pop_nwh/jr_totrace;
    gen jr_m_pop_fbl=pop_nbl/jr_totrace;
    gen jr_m_pop_fin=pop_nin/jr_totrace;
    gen jr_m_pop_fas=pop_nas/jr_totrace;
    gen jr_m_pop_fhi=pop_nhi/jr_totrace;
    gen jr_m_pop_fot=pop_not/jr_totrace;
    foreach g in wh bl in as hi ot {;
     gen jr_m_pop_f`g'2=jr_m_pop_f`g'^2;
     replace jr_m_pop_f`g'2=0 if jr_m_pop_f`g'==.;
     };
    gen jr_m_herfrace=(jr_m_pop_fwh2 + jr_m_pop_fbl2 + jr_m_pop_fin2 + jr_m_pop_fas2 + jr_m_pop_fhi2 + jr_m_pop_fot2);
  *Create educational attainment variables;
   egen ed_toted=rsum(ed_lths ed_hs ed_scol ed_ba);
   gen m_ed_lths=(ed_lths)/ed_toted;
   gen m_ed_hs  =(ed_hs  )/ed_toted;
   gen m_ed_scol=(ed_scol)/ed_toted;
   gen m_ed_ba  =(ed_ba  )/ed_toted;
   drop ed_lths ed_hs ed_scol ed_ba;
   gen m_herfed=(m_ed_lths^2+m_ed_hs^2+m_ed_scol^2+m_ed_ba^2);
  *Create mean income and Gini.  Do mean income two ways: from aggregate income, and from income dist;
   egen nhh=rsum(P80_1-P80_25);
   gen m_lnmeanincA=log((P81_1+P81_2)/nhh);
   *CH algorithm:  Use 4K for lowest (0-5K) bin, and 175K for highest.  Do everything in thousands.;
    gen ch_m_inc_bin1 =P80_1*4;
    gen ch_m_inc_bin2 =P80_2*7.5  ;
    gen ch_m_inc_bin3 =P80_3*11.25;
    gen ch_m_inc_bin4 =P80_4*13.75;
    gen ch_m_inc_bin5 =P80_5*16.25;
    gen ch_m_inc_bin6 =P80_6*18.75;
    gen ch_m_inc_bin7 =P80_7*21.25;
    gen ch_m_inc_bin8 =P80_8*23.75;
    gen ch_m_inc_bin9 =P80_9*26.25;
    gen ch_m_inc_bin10=P80_10*28.75;
    gen ch_m_inc_bin11=P80_11*31.25;
    gen ch_m_inc_bin12=P80_12*33.75;
    gen ch_m_inc_bin13=P80_13*36.25;
    gen ch_m_inc_bin14=P80_14*38.75;
    gen ch_m_inc_bin15=P80_15*41.25;
    gen ch_m_inc_bin16=P80_16*43.75;
    gen ch_m_inc_bin17=P80_17*46.25;
    gen ch_m_inc_bin18=P80_18*48.75;
    gen ch_m_inc_bin19=P80_19*52.5 ;
    gen ch_m_inc_bin20=P80_20*57.5 ;
    gen ch_m_inc_bin21=P80_21*67.5 ;
    gen ch_m_inc_bin22=P80_22*87.5 ;
    gen ch_m_inc_bin23=P80_23*112.5;
    gen ch_m_inc_bin24=P80_24*137.5;
    gen ch_m_inc_bin25=P80_25*175;
    gen ch_m_cuminc_bin1=ch_m_inc_bin1;
    gen ch_m_cumhh_bin1=P80_1;
    foreach i of numlist 2/25 {;
     local j=`i'-1;
     gen ch_m_cuminc_bin`i'=ch_m_cuminc_bin`j'+ch_m_inc_bin`i' if ch_m_inc_bin`i'~=.;
     gen ch_m_cumhh_bin`i'=ch_m_cumhh_bin`j'+P80_`i' if P80_`i'~=.;
     };
    gen ch_m_lnmeanincB=log(ch_m_cuminc_bin25/ch_m_cumhh_bin25);
    foreach i of numlist 1/25 {;
     gen ch_m_cumfinc_bin`i'=ch_m_cuminc_bin`i'/ch_m_cuminc_bin25;
     gen ch_m_cumfhh_bin`i'=ch_m_cumhh_bin`i'/ch_m_cumhh_bin25;
     };
    gen ch_m_gini=ch_m_cumfinc_bin1*ch_m_cumfhh_bin1*0.5;
    foreach i of numlist 2/25 {;
     local j=`i'-1;
     replace ch_m_gini=ch_m_gini+((ch_m_cumfhh_bin`i'-ch_m_cumfhh_bin`j')*(ch_m_cumfinc_bin`j'));
     replace ch_m_gini=ch_m_gini+((ch_m_cumfhh_bin`i'-ch_m_cumfhh_bin`j')*(ch_m_cumfinc_bin`i'-ch_m_cumfinc_bin`j')*0.5);
     };
    replace ch_m_gini=(0.5-ch_m_gini)/0.5;
    drop ch_m_inc_bin* ch_m_cuminc_bin* ch_m_cumhh_bin* ch_m_cumfinc_bin* ch_m_cumfhh_bin*;

   *JR algorithm:  Use 2.5K for lowest, and average of topcoded for highest. (the latter matters for Gini);
    *  Also compute mean log inc, assigning each family the average of the logs of the endpoints of the bins;
    *  My gini algorithm looks different, but amounts to the same thing;
    gen jr_m_inc_bin1 =P80_1*2500;
    gen jr_m_inc_bin2 =P80_2*7500;
    gen jr_m_inc_bin3 =P80_3*11250;
    gen jr_m_inc_bin4 =P80_4*13750;
    gen jr_m_inc_bin5 =P80_5*16250;
    gen jr_m_inc_bin6 =P80_6*18750;
    gen jr_m_inc_bin7 =P80_7*21250;
    gen jr_m_inc_bin8 =P80_8*23750;
    gen jr_m_inc_bin9 =P80_9*26250;
    gen jr_m_inc_bin10=P80_10*28750;
    gen jr_m_inc_bin11=P80_11*31250;
    gen jr_m_inc_bin12=P80_12*33750;
    gen jr_m_inc_bin13=P80_13*36250;
    gen jr_m_inc_bin14=P80_14*38750;
    gen jr_m_inc_bin15=P80_15*41250;
    gen jr_m_inc_bin16=P80_16*43750;
    gen jr_m_inc_bin17=P80_17*46250;
    gen jr_m_inc_bin18=P80_18*48750;
    gen jr_m_inc_bin19=P80_19*52500;
    gen jr_m_inc_bin20=P80_20*57500;
    gen jr_m_inc_bin21=P80_21*67500;
    gen jr_m_inc_bin22=P80_22*87500;
    gen jr_m_inc_bin23=P80_23*112500;
    gen jr_m_inc_bin24=P80_24*137500;
    gen jr_m_inc_bin25=P80_25*(P81_2/P80_25);
    gen jr_m_linc_bin1 =P80_1*log(2500);
    gen jr_m_linc_bin2 =P80_2*(log(5000 )+log(10000))/2;
    gen jr_m_linc_bin3 =P80_3*(log(10000)+log(12500))/2;
    gen jr_m_linc_bin4 =P80_4*(log(12500)+log(15000))/2;
    gen jr_m_linc_bin5 =P80_5*(log(15000)+log(17500))/2;
    gen jr_m_linc_bin6 =P80_6*(log(17500)+log(20000))/2;
    gen jr_m_linc_bin7 =P80_7*(log(20000)+log(22500))/2;
    gen jr_m_linc_bin8 =P80_8*(log(22500)+log(25000))/2;
    gen jr_m_linc_bin9 =P80_9*(log(25000)+log(27500))/2;
    gen jr_m_linc_bin10=P80_10*(log(27500)+log(30000))/2;
    gen jr_m_linc_bin11=P80_11*(log(30000)+log(32500))/2;
    gen jr_m_linc_bin12=P80_12*(log(32500)+log(35000))/2;
    gen jr_m_linc_bin13=P80_13*(log(35000)+log(37500))/2;
    gen jr_m_linc_bin14=P80_14*(log(37500)+log(40000))/2;
    gen jr_m_linc_bin15=P80_15*(log(40000)+log(42500))/2;
    gen jr_m_linc_bin16=P80_16*(log(42500)+log(45000))/2;
    gen jr_m_linc_bin17=P80_17*(log(45000)+log(47500))/2;
    gen jr_m_linc_bin18=P80_18*(log(47500)+log(50000))/2;
    gen jr_m_linc_bin19=P80_19*(log(50000)+log(55000))/2;
    gen jr_m_linc_bin20=P80_20*(log(55000)+log(60000))/2;
    gen jr_m_linc_bin21=P80_21*(log(60000)+log(75000))/2;
    gen jr_m_linc_bin22=P80_22*(log(75000)+log(100000))/2;
    gen jr_m_linc_bin23=P80_23*(log(100000)+log(125000))/2;
    gen jr_m_linc_bin24=P80_24*(log(125000)+log(150000))/2;
    gen jr_m_linc_bin25=P80_25*log(P81_2/P80_25);
    gen jr_m_cuminc_bin1=jr_m_inc_bin1;
    replace jr_m_cuminc_bin1=0 if jr_m_inc_bin1==.;
    gen jr_m_cumlinc_bin1=jr_m_linc_bin1;
    replace jr_m_cumlinc_bin1=0 if jr_m_linc_bin1==.;
    gen jr_m_cumhh_bin1=P80_1;
    replace jr_m_cumhh_bin1=0 if jr_m_cumhh_bin1==.;
    foreach i of numlist 2/25 {;
     local j=`i'-1;
     gen jr_m_cuminc_bin`i'=jr_m_cuminc_bin`j'+jr_m_inc_bin`i' if jr_m_inc_bin`i'~=.;
     replace jr_m_cuminc_bin`i'=jr_m_cuminc_bin`j' if jr_m_inc_bin`i'==.;
     gen jr_m_cumlinc_bin`i'=jr_m_cumlinc_bin`j'+jr_m_linc_bin`i' if jr_m_linc_bin`i'~=.;
     replace jr_m_cumlinc_bin`i'=jr_m_cumlinc_bin`j' if jr_m_linc_bin`i'==.;
     gen jr_m_cumhh_bin`i'=jr_m_cumhh_bin`j'+P80_`i' if P80_`i'~=.;
     replace jr_m_cumhh_bin`i'=jr_m_cumhh_bin`j' if P80_`i'==.;
     };
    gen jr_m_lnmeanincB=log(jr_m_cuminc_bin25/jr_m_cumhh_bin25);
    gen jr_m_meanlninc=jr_m_cumlinc_bin25/jr_m_cumhh_bin25;
    foreach i of numlist 1/25 {;
     gen jr_m_cumfinc_bin`i'=jr_m_cuminc_bin`i'/jr_m_cuminc_bin25;
     gen jr_m_cumfhh_bin`i'=jr_m_cumhh_bin`i'/jr_m_cumhh_bin25;
     };
    gen jr_m_gini=0;
    foreach i of numlist 1/24 {;
     local j=`i'+1;
     replace jr_m_gini=jr_m_gini+(jr_m_cumfhh_bin`i'*jr_m_cumfinc_bin`j') - (jr_m_cumfhh_bin`j'*jr_m_cumfinc_bin`i');
     };
    drop jr_m_inc_bin* jr_m_cuminc_bin* jr_m_cumhh_bin* jr_m_cumfinc_bin* jr_m_cumfhh_bin* jr_m_linc_bin* jr_m_cumlinc_bin*;
  *Create ethnic heterogeneity index;
   *CH adds other hispanic to south american.  Canadians count as british, but french canadians as french.;
    *"Other white" includes subsaharan african, west indian, and other/unclassified.  "Race/Hisp" is excluded.;
    gen ch_m_anch_samer=anch_colo+anch_ecua+anch_peru+anch_otsa+anch_oth;
    gen ch_m_anch_camer=anch_domn+anch_guat+anch_hond+anch_nica+anch_pana+anch_salv+anch_otca;
    gen ch_m_anchf_mex  =     anch_mex  /(anch_mex+anch_pr+anch_cubn+ch_m_anch_samer+ch_m_anch_camer);
    gen ch_m_anchf_pr   =     anch_pr   /(anch_mex+anch_pr+anch_cubn+ch_m_anch_samer+ch_m_anch_camer);
    gen ch_m_anchf_cubn =     anch_cubn /(anch_mex+anch_pr+anch_cubn+ch_m_anch_samer+ch_m_anch_camer);
    gen ch_m_anchf_samer=ch_m_anch_samer/(anch_mex+anch_pr+anch_cubn+ch_m_anch_samer+ch_m_anch_camer);
    gen ch_m_anchf_camer=ch_m_anch_camer/(anch_mex+anch_pr+anch_cubn+ch_m_anch_samer+ch_m_anch_camer);
    gen ch_m_herfhisp=ch_m_anchf_mex^2+ch_m_anchf_pr^2+ch_m_anchf_cubn^2+ch_m_anchf_samer^2+ch_m_anchf_camer^2;
    gen ch_m_anc_brit=anc_engl+anc_sceir+anc_scott+anc_welsh+anc_canad;
    gen ch_m_anc_scand=anc_dansh+anc_finn+anc_norw+anc_swed;
    gen ch_m_anc_russ=anc_russ+anc_ukr;
    gen ch_m_anc_eeur=anc_czech+anc_lith+anc_roman+anc_slov+anc_yugo;
    gen ch_m_anc_benelux=anc_belg+anc_dutch;
    gen ch_m_anc_swisaus=anc_swiss+anc_aust;
    gen ch_m_anc_french=anc_frnch+anc_frcan;
    gen ch_m_anc_otwhite=anc_afric+anc_usa+anc_windn+anc_otr+anc_uncl;
    gen ch_m_totwh=ch_m_anc_brit+ch_m_anc_scand+ch_m_anc_russ+ch_m_anc_eeur+ch_m_anc_benelux+ch_m_anc_swisaus+
                   ch_m_anc_french+ch_m_anc_otwhite+anc_arab+anc_germ+anc_grk+anc_hung+
                   anc_eir+anc_ital+anc_pole+anc_port;
    gen ch_m_herfwhite=0;
    foreach g in brit scand russ eeur benelux swisaus french otwhite {;
     replace ch_m_herfwhite=ch_m_herfwhite+(ch_m_anc_`g'/ch_m_totwh)^2;
     };
    foreach g in arab germ grk hung eir ital pole port {;
     replace ch_m_herfwhite=ch_m_herfwhite+(anc_`g'/ch_m_totwh)^2;
     };
    gen ch_m_herfethn=(ch_m_pop_fwh^(3-ch_m_herfwhite) + ch_m_pop_fbl^2 + ch_m_pop_fin^2 + ch_m_pop_fas^2 +
                       ch_m_pop_fhi^(3-ch_m_herfhisp));
   *JR:  Include other hispanic as a separate category.  Canadians and french canadians are "other",
    *and subsaharan african, USA, west indian, "Race or hispanic origin groups", and unclassified are excluded.;
    egen jr_m_anch_samer=rsum(anch_colo anch_ecua anch_peru anch_otsa);
    egen jr_m_anch_camer=rsum(anch_domn anch_guat anch_hond anch_nica anch_pana anch_salv anch_otca);
    egen jr_m_toteth_h=rsum(anch_mex anch_pr anch_cubn jr_m_anch_samer jr_m_anch_camer anch_oth);
    gen jr_m_anchf_mex  =   anch_mex    /jr_m_toteth_h;
    gen jr_m_anchf_pr   =   anch_pr     /jr_m_toteth_h;
    gen jr_m_anchf_cubn =   anch_cubn   /jr_m_toteth_h;
    gen jr_m_anchf_samer=jr_m_anch_samer/jr_m_toteth_h;
    gen jr_m_anchf_camer=jr_m_anch_camer/jr_m_toteth_h;
    gen jr_m_anchf_oth  =   anch_oth    /jr_m_toteth_h;
    foreach g in mex pr cubn samer camer oth {;
     gen jr_m_anchf2_`g'=jr_m_anchf_`g'^2;
     };
    egen jr_m_herfhisp=rsum(jr_m_anchf2_mex jr_m_anchf2_pr jr_m_anchf2_cubn jr_m_anchf2_samer jr_m_anchf2_camer jr_m_anchf2_oth);
    egen jr_m_anc_brit=rsum(anc_engl anc_sceir anc_scott anc_welsh);
    egen jr_m_anc_scand=rsum(anc_dansh anc_finn anc_norw anc_swed);
    egen jr_m_anc_russ=rsum(anc_russ anc_ukr);
    egen jr_m_anc_eeur=rsum(anc_czech anc_lith anc_roman anc_slov anc_yugo);
    egen jr_m_anc_benelux=rsum(anc_belg anc_dutch);
    egen jr_m_anc_swisaus=rsum(anc_swiss anc_aust);
    egen jr_m_anc_otwhite=rsum(anc_canad anc_frcan anc_otr);
    egen jr_m_totwh=rsum(jr_m_anc_brit jr_m_anc_scand jr_m_anc_russ jr_m_anc_eeur jr_m_anc_benelux jr_m_anc_swisaus
                         anc_frnch jr_m_anc_otwhite anc_arab anc_germ anc_grk anc_hung
                         anc_eir anc_ital anc_pole anc_port);
    gen jr_m_herfwhite=0;
    foreach g in brit scand russ eeur benelux swisaus otwhite {;
     replace jr_m_herfwhite=jr_m_herfwhite+(jr_m_anc_`g'/jr_m_totwh)^2 if jr_m_anc_`g'<.;
     };
    foreach g in frnch arab germ grk hung eir ital pole port {;
     replace jr_m_herfwhite=jr_m_herfwhite+(anc_`g'/jr_m_totwh)^2 if anc_`g'<.;
     };
    gen jr_m_pop_fwh3=jr_m_pop_fwh^(3-jr_m_herfwhite);
    replace jr_m_pop_fwh3=0 if jr_m_pop_fwh==.;
    gen jr_m_pop_fhi3=jr_m_pop_fhi^(3-jr_m_herfhisp);
    replace jr_m_pop_fhi3=0 if jr_m_pop_fhi==.;
    gen jr_m_herfethn=(jr_m_pop_fwh3 + jr_m_pop_fbl2 + jr_m_pop_fin2 + jr_m_pop_fas2 + jr_m_pop_fhi3 + jr_m_pop_fot2);
  *Create fraction private;
   gen m_frpriv=n_priv/(n_pub+n_priv);
  keep fipsmsa ch_m_divis* jr_m_divis* m_pop_n m_arealand
       m_ed_lths m_ed_hs m_ed_scol m_ed_ba m_herfed m_lnmeanincA
       ch_m_pop_fwh ch_m_pop_fbl ch_m_pop_fhi ch_m_pop_fas ch_m_pop_fin ch_m_herfrace
       ch_m_herfhisp ch_m_herfwhite ch_m_herfethn
       ch_m_lnmeanincB ch_m_gini
       jr_m_pop_fwh jr_m_pop_fbl jr_m_pop_fin jr_m_pop_fas jr_m_pop_fhi jr_m_pop_fot jr_m_herfrace
       jr_m_herfhisp jr_m_herfwhite jr_m_herfethn
       jr_m_lnmeanincB jr_m_meanlninc jr_m_gini
       m_frpriv
       ;
  label var fipsmsa       "MSA/PMSA code";
  label var m_pop_n         "MSA total population";
  label var m_arealand      "MSA land area (sq. miles)";
  label var m_ed_lths       "MSA: fr. of adult pop., LTHS";
  label var m_ed_hs         "MSA: fr. of adult pop., HS";
  label var m_ed_scol       "MSA: fr. of adult pop., some college";
  label var m_ed_ba         "MSA: fr. of adult pop., BA+";
  label var m_lnmeanincA    "MSA: log(mean HH inc), from agg inc";
  label var m_frpriv        "MSA: Private enroll share, El+HS";
  label var ch_m_pop_fwh    "MSA pop: Fr. white (CH rule)";
  label var ch_m_pop_fbl    "MSA pop: Fr. black (CH rule)";
  label var ch_m_pop_fhi    "MSA pop: Fr. hisp (CH rule)";
  label var ch_m_pop_fas    "MSA pop: Fr. asian (CH rule)";
  label var ch_m_pop_fin    "MSA pop: Fr. indian (CH rule)";
  label var ch_m_herfrace   "MSA: Racial herfindahl (CH rule)";
  label var ch_m_herfhisp   "MSA: Ethnic herf. among hispanics (CH rule)";
  label var ch_m_herfwhite  "MSA: Ethnic herf. among whites (CH rule)";
  label var ch_m_herfethn   "MSA: Ethnic-adj. racial herf. (CH rule)";
  label var ch_m_lnmeanincB "MSA: log(mean HH inc), from bins (CH rule)";
  label var ch_m_gini       "MSA: Gini coefficient (CH rule)";
  label var jr_m_pop_fwh    "MSA pop: Fr. white (JR rule)";
  label var jr_m_pop_fbl    "MSA pop: Fr. black (JR rule)";
  label var jr_m_pop_fhi    "MSA pop: Fr. hisp (JR rule)";
  label var jr_m_pop_fas    "MSA pop: Fr. asian (JR rule)";
  label var jr_m_pop_fin    "MSA pop: Fr. indian (JR rule)";
  label var jr_m_pop_fot    "MSA pop: Fr. other race (JR rule)";
  label var jr_m_herfrace   "MSA: Racial herfindahl (JR rule)";
  label var jr_m_herfhisp   "MSA: Ethnic herf. among hispanics (JR rule)";
  label var jr_m_herfwhite  "MSA: Ethnic herf. among whites (JR rule)";
  label var jr_m_herfethn   "MSA: Ethnic-adj. racial herf. (JR rule)";
  label var jr_m_lnmeanincB "MSA: log(mean HH inc), from bins (JR rule)";
  label var jr_m_meanlninc  "MSA: mean(log HH inc), from bins";
  label var jr_m_gini       "MSA: Gini coefficient (JR rule)";
  *Compare ch and jr variables;
   foreach v in pop_fwh pop_fbl pop_fhi pop_fas pop_fin herfrace herfhisp herfwhite herfethn gini
                divis1 divis2 divis3 divis4 divis5 divis6 divis7 divis8 divis9 {;
    di "Comparing CH and JR: m`v'";
    su ch_m_`v' jr_m_`v';
    su ch_m_`v' jr_m_`v' if ch_m_`v'~=. & jr_m_`v'~=.;
    corr ch_m_`v' jr_m_`v';
    };
   di "Comparing CH and JR income";
   su m_lnmeanincA ch_m_lnmeanincB jr_m_lnmeanincB jr_m_meanlninc;
   pwcorr m_lnmeanincA ch_m_lnmeanincB jr_m_lnmeanincB jr_m_meanlninc;
  d;
  tempfile stf3a_msadata;
  sort fipsmsa;
  duplicates report fipsmsa;
  compress;
  save `stf3a_msadata';

**Section C:  Read rivers variables and compute MSA-level rivers counts;
di in red "**Section C:  Read rivers variables and compute MSA-level rivers counts";
 *Read Hoxbys streams data sets;
  *Read Hoxbys total streams, looping over states;
  *Note that this data set contains county codes only for stream mouths;
   ! c:\progra~1\winzip\wzunzip ${constructdir_public}\gnis\state_deci_files.zip .\;
   foreach s in AK AL AR AZ CA CO CT DC DE FL GA HI IA ID IL IN KS KY LA MA MD
                ME MI MN MO MS MT NC ND NE NH NJ NM NV NY OH OK OR PA RI SC SD
                TN TX UT VA VT WA WI WV WY {;
    drop _all;
    insheet stabbr fea_name fea_type cntyname fipsst fipscnty primlat primlong primlatdd
            primlongdd sourlat sourlong sourlatdd sourlongdd elevation est_pop fed_stat cellname
            using `s'_deci.txt, comma;
    keep if fea_type=="stream";
    keep if primlatdd~=. & primlongdd~=. & sourlatdd~=. & sourlongdd~=.;
    gen streamlen=sqrt((69.1 * (primlatdd - sourlatdd)) * (69.1 * (primlatdd - sourlatdd)) +
                         (69.1 * (primlongdd - sourlongdd)* cos(sourlatdd/57.3)) *
                         (69.1 * (primlongdd - sourlongdd)* cos(sourlatdd/57.3)));
    keep fea_name fipsst fipscnty streamlen;
    tempfile ch_rivs_`s';
    save `ch_rivs_`s'';
    erase `s'_deci.txt;
    };
   use `ch_rivs_AK';
   foreach s in AL AR AZ CA CO CT DC DE FL GA HI IA ID IL IN KS KY LA MA MD
                ME MI MN MO MS MT NC ND NE NH NJ NM NV NY OH OK OR PA RI SC SD
                TN TX UT VA VT WA WI WV WY {;
    append using `ch_rivs_`s'';
    };
   rename fipsst stfips;
   rename fipscnty cofips;
   sort stfips cofips;
   tempfile ch_strbase;
   save `ch_strbase';
  *Read Hoxbys larger streams;
   drop _all;
   insheet using ${constructdir_public}/misc/mlargerstreams.dat;
   di in red "Three MSA codes in Hoxbys list are invalid";
    list if fipsmsa==2285 | fipsmsa==3755 | fipsmsa==7960;
    *2285 is the 1983 code for E. St. Louis/Bellevue, which in 1990 is part of St. Louis;
    *Hoxby has zero count anyway;
     drop if fipsmsa==2285;
    *3755 is 1983 code for Kansas City, KS PMSA, in 1990 part of Kansas City MO/KS MSA;
     replace mnumrivr=mnumrivr+19 if fipsmsa==3760;
     drop if fipsmsa==3755;
    *7960 is 1980 code for Springfield, OH SMSA.
    *One of the two counties in this SMSA is in Dayton-Springfield MSA in 1990;
    *Assign the one river to that one;
     replace mnumrivr=mnumrivr+1 if fipsmsa==2000;
     drop if fipsmsa==7960;
   sort fipsmsa;
   duplicates report fipsmsa;
   tempfile ch_bigstr;
   save `ch_bigstr';

  *Make an MSA-level data set of Hoxbys streams variables;
  *Hoxbys algorithm is to match to MSAs, then within each msa,
  *drop the stream if the longest segment (with the same name) is <=1 mile.;
   *Grab county-MSA correspondence;
    use `co2msa';
    keep stfips cofips ch_assocmsa;
    *Fix Miami-Dade county code, which is outdated;
     replace cofips=086 if cofips==25 & stfips==12;
    *Keep only one obs per county in NE;
     gsort stfips cofips -ch_assocmsa;
     duplicates report stfips cofips;
     duplicates report stfips cofips if ch_assocmsa<.;
     by stfips cofips: keep if _n==1;
    sort stfips cofips;
   *Merge with streams data;
    merge stfips cofips using `ch_strbase', uniqmaster _merge(str2county);
    tab str2county, missing;
    gen stcofips=stfips*1000+cofips;
    di in red "Note:  64 counties with no streams.  Many are in North Dakota (?), Texas, and independent cities in VA";
    tab stcofips if str2county==1;
    di in red "Note: 388 streams in three nonexistent counties.";
    di "2068 is Denali, Alaska, which is a census area but not a legal borough.  I dont know why it isnt in STF-3A"
       _newline "2232 and 2282 just dont exist, but seem unlikely to be in Anchorage, the only MSA in Alaska";
    tab stcofips if str2county==2;
    drop if str2county~=3;
    drop if ch_assocmsa==. | ch_assocmsa==0;
   *Keep only longest stream with each name in MSA, and drop even this if less than one mile;
    gsort ch_assocmsa fea_name -streamlen;
    by ch_assocmsa fea_name: keep if _n==1;
    drop if streamlen<=1 | streamlen==.;
   *Count all streams that remain;
    collapse (count) ch_nstreams_tot=streamlen, by(ch_assocmsa);
    rename ch_assocmsa fipsmsa;
  *Merge larger and smaller streams together;
   sort fipsmsa;
   merge fipsmsa using `ch_bigstr', unique _merge(ch_str_big2little);
   rename mnumrivr ch_nstreams_lg;
  *Merge on MSA names;
   sort fipsmsa;
   merge fipsmsa using `msanames', unique _merge(ch_str2manames);
   tab ch_str2manames;
   *Which MSAs didnt appear in both data sets?;
    di in red "Not all MSAs have both larger and smaller streams.  Exclude from ch_1, include as zero in ch_2";
    tab ch_str_big2little ch_str2manames, missing;
    di "Several MSAs (in New England) have neither";
    list fipsmsa msaname ch_str_big2little ch_str2manames if ch_nstreams_lg==. & ch_nstreams_tot==., clean;
    di "Others have smaller but not larger";
    list fipsmsa msaname ch_str_big2little ch_str2manames if ch_nstreams_lg==. & ch_nstreams_tot~=., clean;
    di "Others have larger but not smaller";
    list fipsmsa msaname ch_str_big2little ch_str2manames if ch_nstreams_lg~=. & ch_nstreams_tot==., clean;
   gen ch_nstreams2_lg=ch_nstreams_lg;
   replace ch_nstreams2_lg=0 if ch_nstreams_lg==.;
   gen ch_nstreams2_tot=ch_nstreams_tot;
   replace ch_nstreams2_tot=0 if ch_nstreams_tot==.;
   gen ch_nstreams_sm=ch_nstreams_tot-ch_nstreams_lg;
   gen ch_nstreams2_sm=ch_nstreams2_tot-ch_nstreams2_lg;
  sort fipsmsa;
  label var ch_nstreams_tot "# of streams (CH alg, CH data), total";
  label var ch_nstreams_sm "# of smaller streams (CH alg, CH data)";
  label var ch_nstreams_lg "# of larger streams (CH alg, CH data)";
  label var ch_nstreams2_tot "# of streams (CH alg, CH data, .->0), total";
  label var ch_nstreams2_sm "# of smaller streams (CH alg, CH data, .->0)";
  label var ch_nstreams2_lg "# of larger streams (CH alg, CH data, .->0)";
  keep fipsmsa ch_nstreams*;
  tempfile ch_streams;
  save `ch_streams';

 *Read alternative GNIS data set.  This is dated July 1999, and includes all counties through which rivers flow;
 *I extracted all streams features from the GNIS database;
  drop _all;
  insheet using "${mydata}/gnis/rivextract.txt", names comma;
  rename nm name;
  rename sc fipslist;
  sort id name;
  duplicates report id;
  *Compute length of river;
   gen streamlen=sqrt(((69.17 * (ht - dt))^2) + ((69.17 * (hl - dl) * cos(dt/57.3))^2))
                 if ht~=. & hl~=. & dt~=. & dl~=.;
   *"Correct" computation--correlates perfectly;
   gen streamlenB=acos((sin(ht/57.3)*sin(dt/57.3)) +
                       (cos(ht/57.3)*cos(dt/57.3)*cos((min(abs(dl-hl), 360-abs(dl-hl))/57.3)))
                       )*57.3*69.17 if ht~=. & hl~=. & dt~=. & dl~=.;
   corr streamlen streamlenB;
  keep id name fipslist streamlen;
  drop if streamlen<1;
  forvalues i=1/14 {;
   gen cocode`i'=(word(fipslist, `i')) if word(fipslist, `i')~="";
   };
  *Some longer rivers have truncated lists.  Fix these by hand.;
  list id name cocode14 if cocode14~="";
  gen str80 fipslist2="29153 29213 05009 05007 05015 05143 05087 05047 29209 29009" if id==78762;
  replace fipslist2="05033 05131 40079 40061 40135 40101 40145 40143 40117 40113 40071 40103 20035" if id==78956;
  gen str80 fipslist3="20191 20173 20155 20159 20009 20145 20047 20057 20055 20069 20093 20075 08099" if id==78956;
  gen str80 fipslist4="08011 08089 08025 08101 08043 08015 08065" if id==78956;
  replace fipslist2="13291 13143 13145 01017 13285" if id==169320;
  replace fipslist2="08123" if id==201759;
  replace fipslist2="05033 05131 40079 40061 40135 40101 40145 40143 40117 40113 40071 40103 20035" if id==201783;
  replace fipslist3="20191 20173 20155 20159 20009 20145 20047 20057 20055 20069 20093 20075 08099" if id==201783;
  replace fipslist4="08011 08089 08025 08101 08043 08015 08065" if id==201783;
  replace fipslist2="42103 34037 36071 36105 42127 36025" if id==217887;
  replace fipslist2="13181 13105 45001 45007 13147" if id==329607;
  replace fipslist2="13145 01081 01017 13285 13149 13077 13045 13121 13097 13135 13117 13139 13137" if id==354236;
  replace fipslist3="13311 13291" if id==354236;
  replace fipslist2="13081" if id==354272;
  replace fipslist2="13035 13159" if id==356443;
  replace fipslist2="13263 13199 13231 13255 13063 13113 13121" if id==358207;
  replace fipslist2="16053 16067 16031 16013 16077 16011 16005 16019 16051 16065 56023" if id==390729;
  replace fipslist2="17203 17123 17155 17011 17099 17063" if id==410769;
  replace fipslist2="21101 18163 18173 18147 21059 21091 18123 21027 21163 18025 18061 21111 18043" if id==425264;
  replace fipslist3="18019 21185 21223 21041 18077 18155 21077 18115 18029 21015 39061 21117 21037" if id==425264;
  replace fipslist4="39025 21191 21023 39015 21161 39001 21135 39145 21089 39087 21019 54099 54011" if id==425264;
  gen str80 fipslist5="39053 54053 39105 54035 54107 39009 39167 54073 54095 39111 54103 54051 39013" if id==425264;
  gen str80 fipslist6="54069 39081 54009 54029 39029 42007 42003" if id==425264;
  replace fipslist2="18171 18045 18157 18015 18017 18103 18169 18069 18179 18001 18075 39107 39037" if id==449850;
  replace fipslist2="19079 19091 19151 19147 19063 19059" if id==465582;
  replace fipslist2="05033 05131 40079 40061 40135 40101 40145 40143 40117 40113 40071 40103 20035" if id==485196;
  replace fipslist3="20191 20173 20155 20159 20009 20145 20047 20057 20055 20069 20093 20075 08099" if id==485196;
  replace fipslist4="08011 08089 08025 08101 08043 08015 08065" if id==485196;
  replace fipslist2="21045 21137" if id==493284;
  replace fipslist2="21129" if id==513130;
  replace fipslist2="21235 21121 21013 21095" if id==517018;
  replace fipslist2="42115 36007 36017 36077 36025" if id==591690;
  replace fipslist2="09007 09011" if id==617998;
  replace fipslist2="27023 27073 27151 27011" if id==659759;
  replace fipslist2="29195 29041 29033 29107 29177 29095 29047 29165 20209 20103 20005 29021 20043" if id==756398;
  replace fipslist3="29003 29087 31147 29005 31127 31131 19071 31025 19129 31153 31055 19155 31177" if id==756398;
  replace fipslist4="19085 31021 19133 31173 19193 31043 46127 31051 46027 31027 46135 31107 46009" if id==756398;
  replace fipslist5="46023 31015 46053 46015 46085 46017 46069 46065 46117 46119 46041 46107 46129" if id==756398;
  replace fipslist6="46031 46021 38085 38029 38059 38015 38065 38055 38057 38025 38061 38053 38105" if id==756398;
  gen str80 fipslist7="30083 30085 30055 30105 30033 30071 30069 30027 30005 30015 30013 30049 30007" if id==756398;
  gen str80 fipslist8="30031" if id==756398;
  replace fipslist2="56029" if id==793609;
  replace fipslist2="31099 31137 31073 31047 31111" if id==835915;
  replace fipslist2="36001 36091 36115 36113 36031 36041" if id==970226;
  replace fipslist2="51067 51161 51770 51775 51121" if id==1025649;
  replace fipslist2="08009" if id==1091369;
  replace fipslist2="40015 40039 40043 40129 40045" if id==1101881;
  replace fipslist2="05033 05131 40079 40061 40135 40101 40145 40143 40117 40113 40071 40103 20035" if id==1101901;
  replace fipslist3="20191 20173 20155 20159 20009 20145 20047 20057 20055 20069 20093 20075 08099" if id==1101901;
  replace fipslist4="08011 08089 08025 08101 08043 08015 08065" if id==1101901;
  replace fipslist2="40067 48033 48077 40141 48485 40065 48487 48197 40057" if id==1101959;
  replace fipslist2="41049 53005 53071 53021 53025 53077 53037 53017 53007 53047 53043 53019 53065" if id==1140014;
  replace fipslist2="13181 13105 45001 45007 13147" if id==1253486;
  replace fipslist2="46109" if id==1253923;
  replace fipslist2="38103" if id==1261082;
  replace fipslist2="48467" if id==1375103;
  replace fipslist2="48183 48423 48459 48499 48467 48379" if id==1380950;
  replace fipslist2="48213 48139 48257 48113" if id==1383201;
  replace fipslist2="48035 48217 48251 48425 48221 48367 48363 48503 48447 48023 48275 48269 48433" if id==1384148;
  replace fipslist2="48083 48095 48399 48081 48335 48415 48033 48115" if id==1384149;
  replace fipslist2="48337 40067 48077 40033 48485 40101 40141 48487 40065 48197 40057" if id==1384276;
  replace fipslist2="48141 35013" if id==1385432;
  replace fipslist2="51029 51003 51125 51011 51009 51031 51680 51019 51163 51023" if id==1488853;
  replace fipslist2="51067 51161 51770 51775 51121" if id==1502110;
  replace fipslist2="53017 53019 53021 53025 53037 53039 53043 53047 53059 53065 53069 53071 53077" if id==1513353;
  replace fipslist2="55085 55125" if id==1580811;
  replace fipslist2="22125 22029 28157 28001 28063 22107 28021 22065 28149 28055 22035 05017 28151" if id==1629903;
  replace fipslist3="05041 28011 05107 28027 28143 05077 05035 28033 47157 05093 47167 47097 47045" if id==1629903;
  replace fipslist4="29155 47095 29143 21075 29133 21105 21039 21007 17003 29201 29031 17181 17077" if id==1629903;
  replace fipslist5="29157 17157 29186 17133 29099 29189 17163 17119 29183 17083 17013 29113 29163" if id==1629903;
  replace fipslist6="17149 29173 29127 17001 29111 17067 29045 19111 17071 19057 19115 17131 17161" if id==1629903;
  replace fipslist7="19139 19163 19045 17195 17015 19097 17085 19061 55043 19043 55023 19005 55123" if id==1629903;
  replace fipslist8="27055 55063 27169 55121 55011 27157 55091 55093 27049 27037 27163 27123 27053" if id==1629903;
  gen str80 fipslist9="27003 27171 27141 27145 27009 27097 27035 27001 27061 27021 27007 27057 27029" if id==1629903;
  count if cocode14~="" & fipslist2=="";
  gen trunc=(cocode14~="");
  drop cocode14;
  *Drop duplicates:  Those with the same county lists;
   gsort name fipslist* -streamlen id;
   by name fipslist*: drop if _n>1;
  forvalues i=14/26 {;
   gen cocode`i'=(word(fipslist2, `i'-13)) if word(fipslist2, `i'-13)~="";
   };
  forvalues i=27/39 {;
   gen cocode`i'=(word(fipslist3, `i'-26)) if word(fipslist3, `i'-26)~="";
   };
  forvalues i=40/52 {;
   gen cocode`i'=(word(fipslist4, `i'-39)) if word(fipslist4, `i'-39)~="";
   };
  forvalues i=53/65 {;
   gen cocode`i'=(word(fipslist5, `i'-52)) if word(fipslist5, `i'-52)~="";
   };
  forvalues i=66/78 {;
   gen cocode`i'=(word(fipslist6, `i'-65)) if word(fipslist6, `i'-65)~="";
   };
  forvalues i=79/91 {;
   gen cocode`i'=(word(fipslist7, `i'-78)) if word(fipslist7, `i'-78)~="";
   };
  forvalues i=92/104 {;
   gen cocode`i'=(word(fipslist8, `i'-91)) if word(fipslist8, `i'-91)~="";
   };
  forvalues i=105/117 {;
   gen cocode`i'=(word(fipslist9, `i'-104)) if word(fipslist9, `i'-104)~="";
   };
  drop fipslist;
  *Separate into 117 files and reassemble;
   tempfile fullrivs;
   save `fullrivs';
   forvalues i=1/117 {;
    use id name cocode`i' streamlen using `fullrivs' if cocode`i'~="";
    rename cocode`i' cocode;
    gen conum=`i';
    tempfile rivs`i';
    save `rivs`i'';
    };
   use `rivs1';
   forvalues i=2/117 {;
    append using `rivs`i'';
    };
  gsort id name conum;
  by id name: gen foreign_mouth=1 if _n==1 & (index(cocode, "MX")>0 | index(cocode, "CA")>0);
  by id name: replace foreign_mouth=0 if _n==1 & foreign_mouth==.;
  by id name: replace foreign_mouth=foreign_mouth[_n-1] if _n>1;
  gsort id name -conum;
  by id name: gen foreign_source=1 if _n==1 & (index(cocode, "MX")>0 | index(cocode, "CA")>0);
  by id name: replace foreign_source=0 if _n==1 & foreign_source==.;
  by id name: replace foreign_source=foreign_source[_n-1] if _n>1;
  drop if cocode=="" | index(cocode, "MX")>0 | index(cocode, "CA")>0;
  gen stfips=int(real(cocode)/1000);
  gen cofips=real(cocode)-1000*stfips;
  drop cocode;
  egen ncounty=max(conum), by(id name);
  label var ncounty "# of counties through which stream flows";
  label var conum "County # for stream (counting from mouth to source)";
  label var foreign_mouth "=1 if mouth of stream in Canada/Mexico";
  label var foreign_source "=1 if source of stream in Canada/Mexico";
  label var streamlen "Length of stream, in miles";
  tempfile jr_streambase;
  save `jr_streambase';

  *Now construct my rivers variables.  Using data set containing all river-county relationships,
  *count using only first county (mouth), and all counties;
  *Assign fractionally to MSAs in New England based on population share;
   *Start by getting the county-MSA correspondence.  Because I want to merge, I need one record per county;
    use `co2msa';
    keep stfips cofips ma_copopshare fipsmsa;
    replace cofips=86 if cofips==25 & stfips==12;
    gsort stfips cofips -ma_copopshare;
    by stfips cofips: gen marank=_n;
    reshape wide ma_copopshare fipsmsa, i(stfips cofips) j(marank);
    sort stfips cofips;
    duplicates report stfips cofips;
    su;
    tempfile jr_co2msa;
    save `jr_co2msa';
   *Now merge rivers on;
    use stfips cofips id name ncounty conum streamlen if stfips>0 & stfips<=56 using `jr_streambase';
    replace cofips=86 if cofips==25 & stfips==12;
    sort stfips cofips id;
    merge stfips cofips using `jr_co2msa', uniqusing _merge(jr_rivs2cntys);
    tab jr_rivs2cntys;
    di in red "Failed matches, JR streams to counties";
     di "27 counties have no streams.  Most are in Texas.  Three are in metro areas:"
        _newline "San Francisco, Midland Texas, and Manassas Park City, VA (GNIS probably uses earlier codes)";
     list stfips cofips fipsmsa1 if jr_rivs2cntys==2;
     di "375 rivers (in 4 counties) on rivers file dont exist in STF-3A."
        _newline "These are: 2068 (Denali, AK), 2232 (AK, doesnt exist), 2282 (AK, doesnt exist)"
        _newline "and 56000 (Wyoming, unclassified).  Drop all of them for lack of alternative";
     gen stcofips=stfips*1000+cofips;
     tab stcofips if jr_rivs2cntys==1;
     drop stcofips;
    keep if jr_rivs2cntys==3;
    drop stfips cofips;
    reshape long ma_copopshare fipsmsa, i(id conum ncounty) j(marank);
    tempfile jr_rivs2msas;
    save `jr_rivs2msas';
   *Now compute two sets of variables.;
    tempfile jr_rivsA jr_rivsB;
    *The first uses only the first county for each river.;
     *According to GNIS (http://geonames.usgs.gov/gnis_users_guide_descripdbs.html#NGNDB),;
     *counties are ordered from the mouth to the source.  The Deci files that Hoxby uses,
     *however, have only the county of the mouth.  (see item 11 of http://geonames.usgs.gov/faqs.html);
     keep if conum==1;
     drop if fipsmsa==.;
     duplicates report id;
     duplicates report id fipsmsa;
     drop if fipsmsa==.;
     gsort id fipsmsa -ma_copopshare;
     by id fipsmsa: keep if _n==1;
     duplicates report id;
     duplicates report id fipsmsa;
     gen intracnty=ma_copopshare if ncounty==1;
     replace intracnty=0 if intracnty==.;
     gen intercnty=ma_copopshare if ncounty>1 & ncounty<.;
     replace intercnty=0 if intercnty==.;
     gen longstr=ma_copopshare if streamlen>=3.5 & streamlen<.;
     replace longstr=0 if longstr==.;
     gen shortstr=ma_copopshare if streamlen<3.5;
     replace shortstr=0 if shortstr==.;
     collapse (sum) jr_nstreamsA_tot=ma_copopshare
                    jr_nstreamsA_intra=intracnty
                    jr_nstreamsA_inter=intercnty
                    jr_nstreamsA_long=longstr
                    jr_nstreamsA_short=shortstr, by(fipsmsa);
     sort fipsmsa;
     label var jr_nstreamsA_tot   "# of stream mouths in MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsA_intra "# of intra-cnty stream mouths in MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsA_inter "# of inter-cnty stream mouths in MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsA_long  "# of stream mouths >3.5 mi. in MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsA_short "# of stream mouths <3.5 mi. in MSA (JR data, pop wgt in NE)";
     save `jr_rivsA';
    *The second uses any counties, and takes the max weight for each river in each MSA;
     use `jr_rivs2msas';
     drop if fipsmsa==.;
     *There are duplicates--we only want one match between each river and each MSA, with the largest weight;
      duplicates report id fipsmsa ma_copopshare;
      gsort id fipsmsa -ma_copopshare;
      by id fipsmsa: keep if _n==1;
      duplicates report id fipsmsa;
     gen intracnty=ma_copopshare if ncounty==1;
     replace intracnty=0 if intracnty==.;
     gen intercnty=ma_copopshare if ncounty>1 & ncounty<.;
     replace intercnty=0 if intercnty==.;
     gen longstr=ma_copopshare if streamlen>=3.5 & streamlen<.;
     replace longstr=0 if longstr==.;
     gen shortstr=ma_copopshare if streamlen<3.5;
     replace shortstr=0 if shortstr==.;
     collapse (sum) jr_nstreamsB_tot=ma_copopshare
                    jr_nstreamsB_intra=intracnty
                    jr_nstreamsB_inter=intercnty
                    jr_nstreamsB_long=longstr
                    jr_nstreamsB_short=shortstr, by(fipsmsa);
     sort fipsmsa;
     label var jr_nstreamsB_tot   "# of streams thru MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsB_intra "# of intra-cnty streams thru MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsB_inter "# of inter-cnty streams thru MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsB_long  "# of streams >3.5 mi. in MSA (JR data, pop wgt in NE)";
     label var jr_nstreamsB_short "# of streams <3.5 mi. in MSA (JR data, pop wgt in NE)";
     save `jr_rivsB';
   *Bring together;
    merge fipsmsa using `jr_rivsA', unique _merge(jr_rivA2B);
    tab jr_rivA2B;
    list fipsmsa jr_rivA2B if jr_rivA2B~=3;
    drop jr_rivA2B;
    su;
    *Merge on names, and report missings;
     sort fipsmsa;
     merge fipsmsa using `msanames', unique _merge(jr_rivs2names);
     gen shortname=substr(ltrim(msaname), 1, 25);
     di "Three MSAs in Texas are missing A definition"
        _newline "These must have no river mouths.  Midland, TX is also missing B--no streams at all.";
     list fipsmsa shortname jr_nstreamsA_tot jr_nstreamsB_tot if jr_nstreamsA_tot==. | jr_nstreamsB_tot==., clean;
    *Reset missings to zero;
     foreach v in tot inter intra long short {;
      replace jr_nstreamsA_`v'=0 if jr_nstreamsA_`v'==.;
      replace jr_nstreamsB_`v'=0 if jr_nstreamsB_`v'==.;
      };
    keep fipsmsa jr_nstreams*;
    sort fipsmsa;
    tempfile jr_rivs;
    save `jr_rivs';

  *Merge two rivers datasets together;
   merge fipsmsa using `ch_streams', unique _merge(jr2ch_streams);
   tab jr2ch_streams;
   list fipsmsa jr2ch_streams if jr2ch_streams~=3;
   drop jr2ch_streams;
   tempfile streams;
   sort fipsmsa;
   save `streams';

**Section D:  Read in CCD agency file and match to MSAs;
di in red "**Section D:  Read in CCD agency file and match to MSAs";
 *Read CCD agency file for 1987-88, 1989-90, 1991-92;
  drop _all;
  tempfile ccd88 ccd90 ccd92;
  *1988;
   infix str leaid 1-7
         int stfips 1-2
         str leaname 22-51
         str city 77-94
         str zip 97-101
         int cofips 122-124
         int ccd_macode 152-155 using ${constructdir_public}/ccd/Ccdsdu87.txt, clear;
   save `ccd88';
  *1990;
   infix str leaid 1-7
         int stfips 1-2
         str leaname 22-51
         str city 77-94
         str zip 97-101
         int cofips 122-124
         int ccd_macode 152-155 using ${constructdir_public}/ccd/Ccdsdu89.dat, clear;
   save `ccd90';
  *1992;
   infix str leaid 1-7
         int stfips 1-2
         str leaname 22-51
         str city 82-99
         str zip 102-106
         int cofips 127-129
         int ccd_macode 157-160 using ${constructdir_public}/ccd/AGENCY91.DAT, clear;
   save `ccd92';

 *Put MSA codes on the CCD;
  *Start by putting together a data set stacking the three years of CCD;
   use `ccd88';
   gen year=1988;
   append using `ccd90';
   replace year=1990 if year==.;
   append using `ccd92';
   replace year=1992 if year==.;
   duplicates report leaid;
   duplicates report leaid year;
   tempfile ccd_stacked;
   save `ccd_stacked';
  *Proceed in two parts:  Using county codes outside of New England,
  *and using town names and zips (and a lot of hand matching) within New England;
   *Start with non-New England;
    *Grab county-MSA correspondence;
     use `co2msa';
     drop if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50;
     keep stfips cofips fipsmsa;
     sort stfips cofips;
     duplicates report stfips cofips;
     tempfile co2msa_nonne;
     save `co2msa_nonne';
    *Grab districts outside of NE;
     use `ccd_stacked';
     drop if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50;
     *Fix a bad county code;
      *Several districts in St. Genevieve County, Missouri seem to have code 29193, not 29186.
      *29193 was the code prior to December 1979;
       replace cofips=186 if stfips==29 & cofips==193;
    *Merge;
     sort stfips cofips leaid year;
     merge stfips cofips using `co2msa_nonne', uniqusing _merge(ccd2cnty_nonne);
     di in red "There are a fair number of districts that dont match.";
     tab ccd2cnty_nonne;
     *Some of the mismatches are fixed in some years of the CCD;
      gsort leaid -ccd2cnty_nonne -year;
      by leaid: replace stfips=stfips[1] if ccd2cnty_nonne==1 & ccd2cnty_nonne[1]==3;
      by leaid: replace cofips=cofips[1] if ccd2cnty_nonne==1 & ccd2cnty_nonne[1]==3;
     *Resort and remerge;
      drop if ccd2cnty_nonne==2;
      drop fipsmsa ccd2cnty_nonne;
      sort stfips cofips leaid year;
      merge stfips cofips using `co2msa_nonne', uniqusing _merge(ccd2cnty_nonne);
     di in red "Still a fair number of remaining districts that dont match.";
     tab ccd2cnty_nonne;
     sort leaid year;
     egen minmerge=min(ccd2cnty_nonne), by(leaid);
     gen stcofips=stfips*1000+cofips;
     di "17 district-years with invalid county codes--almost all are DOE/Puerto Rico/Guam, with a couple in Alaska";
      list stcofips leaid year leaname ccd2cnty_nonne if minmerge==1;
      drop if minmerge==1;
     di "11 counties without districts"
        _newline "Four are in NYC.  4 are in Hawaii--only Honolulu has a district, which is fine, since it is the MSA"
        _newline "One is Yellowstone National Park, Montana.  One is Loving County, Texas.  One is Williamsburg city, VA."
        _newline "Drop all.";
      list stcofips fipsmsa if ccd2cnty_nonne==2;
      drop if ccd2cnty_nonne==2;
     keep leaid year fipsmsa ccd_macode;
     tempfile ccd2msa_nonne;
     save `ccd2msa_nonne';
   *Now New England.  This is much harder.;
    *Start by separating districts in counties that are entirely non-metropolitan;
     *Get list of counties in MSAs in New England;
      use `co2msa';
      keep if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50;
      keep stfips cofips fipsmsa ma_copopshare;
      gsort stfips cofips fipsmsa;
      by stfips cofips: keep if _n==1;
      gen nonmsa=(fipsmsa==.);
      keep stfips cofips nonmsa;
      sort stfips cofips;
      duplicates report stfips cofips;
      tempfile co2msa_ne;
      save `co2msa_ne';
     *Get districts in New England;
      use `ccd_stacked';
      keep if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50;
      *Fix a bad county code--this district is fine in another year;
       replace cofips=17 if leaid=="3300013" & cofips==8;
      *Another district has a bad county code every year, but has address in Jaffrey, NH, which is Cheshire county;
       replace cofips=5 if leaid=="3300012";
      sort stfips cofips leaid year;
     *Merge together, and separate by county type;
      merge stfips cofips using `co2msa_ne', uniqusing _merge(ccd2cnty_ne);
      *All merge correctly;
      tab ccd2cnty_ne;
      tempfile nedist nedist_nonmsa nedist_msacnty;
      save `nedist';
      keep if nonmsa==1;
      keep leaid year ccd_macode;
      gen fipsmsa=.;
      save `nedist_nonmsa';
      use `nedist';
      drop if nonmsa==1;
      drop ccd2cnty_ne nonmsa;
      save `nedist_msacnty';
    *Now, need to work on districts in metropolitan counties;
          *This code creates a list, to use in hand-matching based on town name;
          *  *Make a list of them;
          *   gsort stfips cofips city zip leaid -year;
          *   outsheet using ccd_ne_msacnty_distlist.txt, replace;
          *  *Also, make a list of towns in those counties;
          *use stfips cofips townfips areaname fipsmsa
          *if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50
          *using tmp_stf3a_town;
          *   *use stfips cofips townfips areaname fipsmsa
          *       if stfips==9 | stfips==23 | stfips==25 | stfips==33 | stfips==44 | stfips==50
          *       using `stf3a_town';
          *   keep stfips cofips townfips areaname fipsmsa;
          *   sort stfips cofips;
          *   merge stfips cofips using `co2msa_ne', uniqusing _merge(stf3a2cnty_ne);
          *   tab stf3a2cnty_ne;
          *   drop if nonmsa==1;
          *   keep stfips cofips townfips areaname fipsmsa;
          *   sort stfips cofips areaname townfips;
          *   outsheet using stf3a_ne_townlist.txt, replace;
     use `nedist_msacnty';
     gen townfips=.;
     *I hand-matched the city names in the relevant district addresses  ;
     *to list of town FIPS codes in the same counties, and created a do ;
     *file to perform these matches.  This file only does exact matches,;
     *where there is no ambiguity.;
      quietly do "makedata.include.ccd2msa.nedists.exacttownmatch.do";
      gen matchqual=1 if townfips>0 & townfips<.;
      replace townfips=. if townfips==0;
          *This code creates a list of districts that did not match, ;
          *to use in hand-matching based on zip and/or district name;
          *   tempfile nedist_townmtch1;
          *   save `nedist_townmtch1';
          *   keep if townfips==.;
          *   gsort stfips cofips leaid year;
          *   outsheet leaid leaname ccd_macode stfips cofips city zip year	using
          *     ccd_ne_msacnty_distlist2.txt, replace;
          *   use `nedist_townmtch1';
     *For those where the town name didnt suffice, I used the district name, then the zip,
     *to assign the district.  In many cases, the district name sufficed.  In several cases,
     *the original county code appeared to be wrong, and was fixed.;
      quietly do "makedata.include.ccd2msa.nedists.backupmatch.do";
     *Check that all districts are assigned;
      tab matchqual, missing;
     *Check that each district is consistently assigned to the same townfips;
      sort leaid year;
      egen lowtown=min(townfips), by(leaid);
      egen hightown=max(townfips), by(leaid);
      gen diff=(lowtown~=hightown);
      tab diff;
     *A fair number of problems;
          *Output to examine by hand;
          *tempfile nedist_townmtch2;
          *save `nedist_townmtch2';
          *keep if diff==1;
          *drop diff;
          *gsort leaid matchqual -year;
          *outsheet using ccd_ne_msacnty_distlist3.txt, replace;
     *Fix inconsistencies, prioritizing LEA name over town name and multiple years over one;
      quietly do "makedata.include.ccd2msa.nedists.inconsistencies.do";
      replace matchqual=12 if diff>0;
      drop lowtown hightown diff;
      sort leaid year;
     *Check again that each district is consistently assigned to the same townfips;
      sort leaid year;
      egen lowtown=min(townfips), by(leaid);
      egen hightown=max(townfips), by(leaid);
      gen diff=(lowtown~=hightown);
      tab diff;
      drop diff;
     *Finally done--merge to MSA list;
      keep leaid year stfips cofips townfips matchqual ccd_macode;
      sort stfips cofips townfips leaid year;
      save `nedist_msacnty', replace;
      use stfips cofips townfips fipsmsa
          using `stf3a_netown';
      sort stfips cofips townfips;
      merge stfips cofips townfips using `nedist_msacnty', uniqmaster _merge(ccd2stf3a_ne);
      tab ccd2stf3a_ne;
      drop if ccd2stf3a_ne==1;
      keep leaid year fipsmsa ccd_macode;
      sort leaid year;
      tempfile ccd2msa_ne;
      save `ccd2msa_ne';
    *Add back districts in non-metropolitan counties;
      use `nedist_nonmsa';
      keep leaid year ccd_macode;
      append using `ccd2msa_ne';
      sort leaid year;
      save `ccd2msa_ne', replace;
   *Bring New England and rest of the country together;
     append using `ccd2msa_nonne';
     *Check to see whether any district changes MSAs over time;
      sort leaid year;
      by leaid: gen diff=(leaid~=leaid[_n-1]) if _n>1;
      tab diff;
      drop diff;
     *Since they dont (by construction in New England--I fixed discrepancies), just keep one per leaid;
      sort leaid year;
      by leaid: keep if _n==1;
      drop year;
      sort leaid;
     tempfile ccd2msa;
     save `ccd2msa';

**Section E:  Make a choice index;
di in red "**Section E:  Make a choice index";
 *Read CCD school files for 1989-90, to get enrollment information by district, for use
 *in constructing the choice index.  Note that Hoxby uses the SDDB for this, which should
 *also describe this school year.  Note:  the numeric variables are read in as strings,
 *as there are many "M"s and "N"s indicating missing value codes (expected or unexpected, respectively).;
  tempfile ccd_skl90 ccd_agenroll90;
  drop _all;
  infix int stfips 1-2
        str leaid  1-7
        int schno  8-12
        int nces_schid 1-12
        str enroll_g8  233-237
        str enroll_g9  238-242
        str enroll_g10 243-247
        str enroll_g11 248-252
        str enroll_g12 253-257
        str enroll_tot 258-263 using ${mydata}/ccd/Raw/School/psu89ai.txt;
  save `ccd_skl90';
  drop _all;
  infix int stfips 1-2
        str leaid  1-7
        int schno  8-12
        int nces_schid 1-12
        str enroll_g8  233-237
        str enroll_g9  238-242
        str enroll_g10 243-247
        str enroll_g11 248-252
        str enroll_g12 253-257
        str enroll_tot 258-263 using ${mydata}/ccd/Raw/School/psu89kn.txt;
  append using `ccd_skl90';
  save `ccd_skl90', replace;
  drop _all;
  infix int stfips 1-2
        str leaid  1-7
        int schno  8-12
        int nces_schid 1-12
        str enroll_g8  233-237
        str enroll_g9  238-242
        str enroll_g10 243-247
        str enroll_g11 248-252
        str enroll_g12 253-257
        str enroll_tot 258-263 using ${mydata}/ccd/Raw/School/psu89ow.txt;
  append using `ccd_skl90';
  save `ccd_skl90', replace;
  sort leaid;
  *Convert the string enrollment numbers to real numbers;
   foreach g in g8 g9 g10 g11 g12 tot {;
    quietly gen enr_`g'=real(enroll_`g') if enroll_`g'~="M" & enroll_`g'~="N" & enroll_`g'~="" & enroll_`g'~=".";
    gen err_`g'=(enr_`g'==. & enroll_`g'~="M" & enroll_`g'~="N" & enroll_`g'~="") & enroll_`g'~=".";
   };
   *How many observations did we not get an enrollment number or a missing-value code for?;
   su err_*;
  *Collapse to make a district-level enrollment variable;
   collapse (sum) enr_*, by(leaid);
   foreach g in g8 g9 g10 g11 g12 tot {;
    rename enr_`g' enroll_`g';
    };
   keep if enroll_g8~=. | enroll_tot~=.;
   replace enroll_g8=0 if enroll_g8==.;
   replace enroll_tot=0 if enroll_tot==.;
   sort leaid;
   duplicates report leaid;
   save `ccd_agenroll90';
 *Merge to CCD-MSA crosswalk, and construct choice index;
  merge leaid using `ccd2msa', unique _merge(lea_enroll2msa);
  tab lea_enroll2msa;
  *Many districts without enrollment--these may be administrative, or may be from other years;
   drop if lea_enroll2msa==2;
  *11 districts dont match to crosswalk;
   list leaid enroll_g8 enroll_tot if lea_enroll2msa==1;
   drop if lea_enroll2msa==1;
   drop lea_enroll2msa;
  sort leaid;
  drop if fipsmsa==. | fipsmsa==0;
  egen msa_enroll_g8=sum(enroll_g8), by(fipsmsa);
  egen msa_enroll_tot=sum(enroll_tot), by(fipsmsa);
  gen sqshare_g8=(enroll_g8/msa_enroll_g8)^2;
  gen sqshare_tot=(enroll_tot/msa_enroll_tot)^2;
  egen herf_g8=sum(sqshare_g8), by(fipsmsa);
  egen herf_tot=sum(sqshare_tot), by(fipsmsa);
  gen ch_choice=1-herf_tot;
  gen jr_choice=1-herf_g8;
  duplicates drop fipsmsa ch_choice jr_choice, force;
  duplicates report fipsmsa;
  sort fipsmsa;
  keep fipsmsa ch_choice jr_choice;
  label var ch_choice "Choice index over all grade enroll (CH rule)";
  label var jr_choice "Choice index over G8 enroll (JR rule)";
  tempfile choice;
  sort fipsmsa;
  save `choice';

**Section F:  Pull together MSA-level data set;
di in red "**Section F:  Pull together MSA-level data set";
 *Pull together STF-3A file, streams file, and choice file;
 use `choice';
 sort fipsmsa;
 merge fipsmsa using `streams', unique _merge(msa_choice2streams);
 sort fipsmsa;
 merge fipsmsa using `stf3a_msadata', unique _merge(msa_2stf3a);
 tab msa_choice2streams msa_2stf3a, missing;
 sort fipsmsa;
 save ${mywork_public}/msadata.dta, replace;
 d;
 su;

**Section G:  Read SDDB and make district-level covariates;
di in red "**Section G:  Read SDDB and make district-level covariates";
 *Read SDDB;
  *NOTE:  SDDB file is horrendously complicated.  ;
  *I read in in SAS, using program getsddb.sas, and work with basedata.dta;
 *Process SDDB.;
 *NOTE: CH provides alternative SDDB data.  I use mine, but could use hers.;
  drop _all;
  use ${mydata}/sddb/basedata.dta;
  rename state d_stfips;
  *Population;
   rename p0010001 d_pop;
  *Education variables;
   rename p1880001 adult_mlths;
   rename p1880002 adult_flths;
   rename p1880003 adult_mhs;
   rename p1880004 adult_fhs;
   rename p1880005 adult_mscol;
   rename p1880006 adult_fscol;
   rename p1880007 adult_mba;
   rename p1880008 adult_fba;
  *Race variables;
   rename p0120001 pop_nwh;
   rename p0120002 pop_nbl;
   rename p0120003 pop_nin;
   rename p0120004 pop_nas;
   rename p0120005 pop_not;
   *Make hispanic;
    egen pop_nhi=rsum(p0120006 p0120007 p0120008 p0120009 p0120010);
  *Ancestry variables--Hispanic ethnicity;
   rename p0110001 anch_nothisp;
   rename p0110002 anch_mex;
   rename p0110003 anch_pr;
   rename p0110004 anch_cubn;
   rename p0110005 anch_domn;
   rename p0110006 anch_guat;
   rename p0110007 anch_hond;
   rename p0110008 anch_nica;
   rename p0110009 anch_pana;
   rename p0110010 anch_salv;
   rename p0110011 anch_otca;
   rename p0110012 anch_colo;
   rename p0110013 anch_ecua;
   rename p0110014 anch_peru;
   rename p0110015 anch_otsa;
   rename p0110016 anch_oth;
  *Ancestry variables--non-Hispanics;
   rename p0330001 anc_arab;
   rename p0330002 anc_aust;
   rename p0330003 anc_belg;
   rename p0330004 anc_canad;
   rename p0330005 anc_czech;
   rename p0330006 anc_dansh;
   rename p0330007 anc_dutch;
   rename p0330008 anc_engl;
   rename p0330009 anc_finn;
   rename p0330010 anc_frnch;
   rename p0330011 anc_frcan;
   rename p0330012 anc_germ;
   rename p0330013 anc_grk;
   rename p0330014 anc_hung;
   rename p0330015 anc_eir;
   rename p0330016 anc_ital;
   rename p0330017 anc_lith;
   rename p0330018 anc_norw;
   rename p0330019 anc_pole;
   rename p0330020 anc_port;
   rename p0330021 anc_roman;
   rename p0330022 anc_russ;
   rename p0330023 anc_sceir;
   rename p0330024 anc_scott;
   rename p0330025 anc_slov;
   rename p0330026 anc_afric;
   rename p0330027 anc_swed;
   rename p0330028 anc_swiss;
   rename p0330029 anc_ukr;
   rename p0330030 anc_usa;
   rename p0330031 anc_welsh;
   rename p0330032 anc_windn;
   rename p0330033 anc_yugo;
   rename p0330034 anc_race;
   rename p0330035 anc_otr;
   rename p0330036 anc_uncl;
  *Start calculating control variables;
   *Start with leaid;
    drop if substr(dist, 1, 2)=="AE" | substr(dist, 1, 2)=="81";
    gen distnum=real(dist);
    drop if distnum<=0 | distnum==.;
    gen leaid=(d_stfips*100000)+distnum;
   *Create race and racial heterogeneity indices;
    *CH algorithm, ignoring other race;
     gen ch_d_pop_fwh=pop_nwh/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
     gen ch_d_pop_fbl=pop_nbl/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
     gen ch_d_pop_fin=pop_nin/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
     gen ch_d_pop_fas=pop_nas/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
     gen ch_d_pop_fhi=pop_nhi/(pop_nwh+pop_nbl+pop_nin+pop_nas+pop_nhi);
     gen ch_d_herfrace=(ch_d_pop_fwh^2 + ch_d_pop_fbl^2 + ch_d_pop_fin^2 + ch_d_pop_fas^2 + ch_d_pop_fhi^2);
    *JR algorithm, including other race;
     egen jr_d_totrace=rsum(pop_nwh pop_nbl pop_nin pop_nas pop_nhi pop_not);
     gen jr_d_pop_fwh=pop_nwh/jr_d_totrace;
     gen jr_d_pop_fbl=pop_nbl/jr_d_totrace;
     gen jr_d_pop_fin=pop_nin/jr_d_totrace;
     gen jr_d_pop_fas=pop_nas/jr_d_totrace;
     gen jr_d_pop_fhi=pop_nhi/jr_d_totrace;
     gen jr_d_pop_fot=pop_not/jr_d_totrace;
     foreach g in wh bl in as hi ot {;
      gen jr_d_pop_f`g'2=jr_d_pop_f`g'^2;
      replace jr_d_pop_f`g'2=0 if jr_d_pop_f`g'==.;
      };
     gen jr_d_herfrace=(jr_d_pop_fwh2 + jr_d_pop_fbl2 + jr_d_pop_fin2 + jr_d_pop_fas2 + jr_d_pop_fhi2 + jr_d_pop_fot2);
   *Create educational attainment variables;
    egen d_ed_toted=rsum(adult_mlths adult_flths adult_mhs adult_fhs
                       adult_mscol adult_fscol adult_mba adult_fba);
    gen d_ed_lths=(adult_mlths+adult_flths)/d_ed_toted;
    gen d_ed_hs  =(adult_mhs  +adult_fhs  )/d_ed_toted;
    gen d_ed_scol=(adult_mscol+adult_fscol)/d_ed_toted;
    gen d_ed_ba  =(adult_mba  +adult_fba  )/d_ed_toted;
    gen d_herfed=(d_ed_lths^2+d_ed_hs^2+d_ed_scol^2+d_ed_ba^2);
   *Create mean income and Gini.  Do mean income two ways: from aggregate income, and from income dist;
    egen d_numhh=rsum(p0800001-p0800025);
    gen d_lnmeanincA=log((p0810001+p0810002)/d_numhh);
    egen d_numhh2=rsum(p0800001-p0800024);
    gen d_lnmeanincA2=log((p0810001)/d_numhh);
    gen d_lnmeanincA3=log((p0810001)/d_numhh2);
    *CH algorithm:  Use 4K for lowest (0-5K) bin, and 175K for highest.  Do everything in thousands.;
     gen ch_d_inc_bin1 =p0800001*4;
     gen ch_d_inc_bin2 =p0800002*7.5  ;
     gen ch_d_inc_bin3 =p0800003*11.25;
     gen ch_d_inc_bin4 =p0800004*13.75;
     gen ch_d_inc_bin5 =p0800005*16.25;
     gen ch_d_inc_bin6 =p0800006*18.75;
     gen ch_d_inc_bin7 =p0800007*21.25;
     gen ch_d_inc_bin8 =p0800008*23.75;
     gen ch_d_inc_bin9 =p0800009*26.25;
     gen ch_d_inc_bin10=p0800010*28.75;
     gen ch_d_inc_bin11=p0800011*31.25;
     gen ch_d_inc_bin12=p0800012*33.75;
     gen ch_d_inc_bin13=p0800013*36.25;
     gen ch_d_inc_bin14=p0800014*38.75;
     gen ch_d_inc_bin15=p0800015*41.25;
     gen ch_d_inc_bin16=p0800016*43.75;
     gen ch_d_inc_bin17=p0800017*46.25;
     gen ch_d_inc_bin18=p0800018*48.75;
     gen ch_d_inc_bin19=p0800019*52.5 ;
     gen ch_d_inc_bin20=p0800020*57.5 ;
     gen ch_d_inc_bin21=p0800021*67.5 ;
     gen ch_d_inc_bin22=p0800022*87.5 ;
     gen ch_d_inc_bin23=p0800023*112.5;
     gen ch_d_inc_bin24=p0800024*137.5;
     gen ch_d_inc_bin25=p0800025*175;
     gen ch_d_cuminc_bin1=ch_d_inc_bin1;
     gen ch_d_cumhh_bin1=p0800001;
     foreach i of numlist 2/25 {;
      local j=`i'-1;
      gen ch_d_cuminc_bin`i'=ch_d_cuminc_bin`j'+ch_d_inc_bin`i' if ch_d_inc_bin`i'~=.;
      if `i'<10 {;
       gen ch_d_cumhh_bin`i'=ch_d_cumhh_bin`j'+p080000`i' if p080000`i'~=.;
       };
      else {;
       gen ch_d_cumhh_bin`i'=ch_d_cumhh_bin`j'+p08000`i' if p08000`i'~=.;
       };
      };
     gen ch_d_lnmeanincB=log(ch_d_cuminc_bin25/ch_d_cumhh_bin25);
     foreach i of numlist 1/25 {;
      gen ch_d_cumfinc_bin`i'=ch_d_cuminc_bin`i'/ch_d_cuminc_bin25;
      gen ch_d_cumfhh_bin`i'=ch_d_cumhh_bin`i'/ch_d_cumhh_bin25;
      };
     gen ch_d_gini=ch_d_cumfinc_bin1*ch_d_cumfhh_bin1*0.5;
     foreach i of numlist 2/25 {;
      local j=`i'-1;
      replace ch_d_gini=ch_d_gini+((ch_d_cumfhh_bin`i'-ch_d_cumfhh_bin`j')*(ch_d_cumfinc_bin`j'));
      replace ch_d_gini=ch_d_gini+((ch_d_cumfhh_bin`i'-ch_d_cumfhh_bin`j')*(ch_d_cumfinc_bin`i'-ch_d_cumfinc_bin`j')*0.5);
      };
     replace ch_d_gini=(0.5-ch_d_gini)/0.5;
     drop ch_d_inc_bin* ch_d_cuminc_bin* ch_d_cumhh_bin* ch_d_cumfinc_bin* ch_d_cumfhh_bin*;

    *JR algorithm:  Use 2.5K for lowest, and average of topcoded for highest. (the latter matters for Gini);
     *  Also compute mean log inc, assigning each family the average of the logs of the endpoints of the bins;
     *  My gini algorithm looks different, but amounts to the same thing;
     gen jr_d_inc_bin1 =p0800001*2500;
     gen jr_d_inc_bin2 =p0800002*7500;
     gen jr_d_inc_bin3 =p0800003*11250;
     gen jr_d_inc_bin4 =p0800004*13750;
     gen jr_d_inc_bin5 =p0800005*16250;
     gen jr_d_inc_bin6 =p0800006*18750;
     gen jr_d_inc_bin7 =p0800007*21250;
     gen jr_d_inc_bin8 =p0800008*23750;
     gen jr_d_inc_bin9 =p0800009*26250;
     gen jr_d_inc_bin10=p0800010*28750;
     gen jr_d_inc_bin11=p0800011*31250;
     gen jr_d_inc_bin12=p0800012*33750;
     gen jr_d_inc_bin13=p0800013*36250;
     gen jr_d_inc_bin14=p0800014*38750;
     gen jr_d_inc_bin15=p0800015*41250;
     gen jr_d_inc_bin16=p0800016*43750;
     gen jr_d_inc_bin17=p0800017*46250;
     gen jr_d_inc_bin18=p0800018*48750;
     gen jr_d_inc_bin19=p0800019*52500;
     gen jr_d_inc_bin20=p0800020*57500;
     gen jr_d_inc_bin21=p0800021*67500;
     gen jr_d_inc_bin22=p0800022*87500;
     gen jr_d_inc_bin23=p0800023*112500;
     gen jr_d_inc_bin24=p0800024*137500;
     gen jr_d_inc_bin25=p0800025*(p0810002/p0800025);
     gen jr_d_linc_bin1 =p0800001*log(2500);
     gen jr_d_linc_bin2 =p0800002*(log(5000 )+log(10000))/2;
     gen jr_d_linc_bin3 =p0800003*(log(10000)+log(12500))/2;
     gen jr_d_linc_bin4 =p0800004*(log(12500)+log(15000))/2;
     gen jr_d_linc_bin5 =p0800005*(log(15000)+log(17500))/2;
     gen jr_d_linc_bin6 =p0800006*(log(17500)+log(20000))/2;
     gen jr_d_linc_bin7 =p0800007*(log(20000)+log(22500))/2;
     gen jr_d_linc_bin8 =p0800008*(log(22500)+log(25000))/2;
     gen jr_d_linc_bin9 =p0800009*(log(25000)+log(27500))/2;
     gen jr_d_linc_bin10=p0800010*(log(27500)+log(30000))/2;
     gen jr_d_linc_bin11=p0800011*(log(30000)+log(32500))/2;
     gen jr_d_linc_bin12=p0800012*(log(32500)+log(35000))/2;
     gen jr_d_linc_bin13=p0800013*(log(35000)+log(37500))/2;
     gen jr_d_linc_bin14=p0800014*(log(37500)+log(40000))/2;
     gen jr_d_linc_bin15=p0800015*(log(40000)+log(42500))/2;
     gen jr_d_linc_bin16=p0800016*(log(42500)+log(45000))/2;
     gen jr_d_linc_bin17=p0800017*(log(45000)+log(47500))/2;
     gen jr_d_linc_bin18=p0800018*(log(47500)+log(50000))/2;
     gen jr_d_linc_bin19=p0800019*(log(50000)+log(55000))/2;
     gen jr_d_linc_bin20=p0800020*(log(55000)+log(60000))/2;
     gen jr_d_linc_bin21=p0800021*(log(60000)+log(75000))/2;
     gen jr_d_linc_bin22=p0800022*(log(75000)+log(100000))/2;
     gen jr_d_linc_bin23=p0800023*(log(100000)+log(125000))/2;
     gen jr_d_linc_bin24=p0800024*(log(125000)+log(150000))/2;
     gen jr_d_linc_bin25=p0800025*log(p0810002/p0800025);
     gen jr_d_cuminc_bin1=jr_d_inc_bin1;
     replace jr_d_cuminc_bin1=0 if jr_d_inc_bin1==.;
     gen jr_d_cumlinc_bin1=jr_d_linc_bin1;
     replace jr_d_cumlinc_bin1=0 if jr_d_linc_bin1==.;
     gen jr_d_cumhh_bin1=p0800001;
     replace jr_d_cumhh_bin1=0 if jr_d_cumhh_bin1==.;
     foreach i of numlist 2/25 {;
      local j=`i'-1;
      gen jr_d_cuminc_bin`i'=jr_d_cuminc_bin`j'+jr_d_inc_bin`i' if jr_d_inc_bin`i'~=.;
      replace jr_d_cuminc_bin`i'=jr_d_cuminc_bin`j' if jr_d_inc_bin`i'==.;
      gen jr_d_cumlinc_bin`i'=jr_d_cumlinc_bin`j'+jr_d_linc_bin`i' if jr_d_linc_bin`i'~=.;
      replace jr_d_cumlinc_bin`i'=jr_d_cumlinc_bin`j' if jr_d_linc_bin`i'==.;
      if `i'<10 {;
       gen jr_d_cumhh_bin`i'=jr_d_cumhh_bin`j'+p080000`i' if p080000`i'~=.;
       replace jr_d_cumhh_bin`i'=jr_d_cumhh_bin`j' if p080000`i'==.;
       };
      else {;
       gen jr_d_cumhh_bin`i'=jr_d_cumhh_bin`j'+p08000`i' if p08000`i'~=.;
       replace jr_d_cumhh_bin`i'=jr_d_cumhh_bin`j' if p08000`i'==.;
       };
      };
     gen jr_d_lnmeanincB=log(jr_d_cuminc_bin25/jr_d_cumhh_bin25);
     gen jr_d_meanlninc=jr_d_cumlinc_bin25/jr_d_cumhh_bin25;
     foreach i of numlist 1/25 {;
      gen jr_d_cumfinc_bin`i'=jr_d_cuminc_bin`i'/jr_d_cuminc_bin25;
      gen jr_d_cumfhh_bin`i'=jr_d_cumhh_bin`i'/jr_d_cumhh_bin25;
      };
     gen jr_d_gini=0;
     foreach i of numlist 1/24 {;
      local j=`i'+1;
      replace jr_d_gini=jr_d_gini+(jr_d_cumfhh_bin`i'*jr_d_cumfinc_bin`j') - (jr_d_cumfhh_bin`j'*jr_d_cumfinc_bin`i');
      };
     drop jr_d_inc_bin* jr_d_cuminc_bin* jr_d_cumhh_bin* jr_d_cumfinc_bin* jr_d_cumfhh_bin* jr_d_linc_bin* jr_d_cumlinc_bin*;
   *Create ethnic heterogeneity index;
    *CH adds other hispanic to south american.  Canadians count as british, but french canadians as french.;
     *"Other white" includes subsaharan african, west indian, and other/unclassified.  "Race/Hisp" is excluded.;
     gen ch_d_anch_samer=anch_colo+anch_ecua+anch_peru+anch_otsa+anch_oth;
     gen ch_d_anch_camer=anch_domn+anch_guat+anch_hond+anch_nica+anch_pana+anch_salv+anch_otca;
     gen ch_d_anchf_mex  =     anch_mex  /(anch_mex+anch_pr+anch_cubn+ch_d_anch_samer+ch_d_anch_camer);
     gen ch_d_anchf_pr   =     anch_pr   /(anch_mex+anch_pr+anch_cubn+ch_d_anch_samer+ch_d_anch_camer);
     gen ch_d_anchf_cubn =     anch_cubn /(anch_mex+anch_pr+anch_cubn+ch_d_anch_samer+ch_d_anch_camer);
     gen ch_d_anchf_samer=ch_d_anch_samer/(anch_mex+anch_pr+anch_cubn+ch_d_anch_samer+ch_d_anch_camer);
     gen ch_d_anchf_camer=ch_d_anch_camer/(anch_mex+anch_pr+anch_cubn+ch_d_anch_samer+ch_d_anch_camer);
     gen ch_d_herfhisp=ch_d_anchf_mex^2+ch_d_anchf_pr^2+ch_d_anchf_cubn^2+ch_d_anchf_samer^2+ch_d_anchf_camer^2;
     gen ch_d_anc_brit=anc_engl+anc_sceir+anc_scott+anc_welsh+anc_canad;
     gen ch_d_anc_scand=anc_dansh+anc_finn+anc_norw+anc_swed;
     gen ch_d_anc_russ=anc_russ+anc_ukr;
     gen ch_d_anc_eeur=anc_czech+anc_lith+anc_roman+anc_slov+anc_yugo;
     gen ch_d_anc_benelux=anc_belg+anc_dutch;
     gen ch_d_anc_swisaus=anc_swiss+anc_aust;
     gen ch_d_anc_french=anc_frnch+anc_frcan;
     gen ch_d_anc_otwhite=anc_afric+anc_usa+anc_windn+anc_otr+anc_uncl;
     gen ch_d_totwh=ch_d_anc_brit+ch_d_anc_scand+ch_d_anc_russ+ch_d_anc_eeur+ch_d_anc_benelux+ch_d_anc_swisaus+
                    ch_d_anc_french+ch_d_anc_otwhite+anc_arab+anc_germ+anc_grk+anc_hung+
                    anc_eir+anc_ital+anc_pole+anc_port;
     gen ch_d_herfwhite=0;
     foreach g in brit scand russ eeur benelux swisaus french otwhite {;
      replace ch_d_herfwhite=ch_d_herfwhite+(ch_d_anc_`g'/ch_d_totwh)^2;
      };
     foreach g in arab germ grk hung eir ital pole port {;
      replace ch_d_herfwhite=ch_d_herfwhite+(anc_`g'/ch_d_totwh)^2;
      };
     gen ch_d_herfethn=(ch_d_pop_fwh^(3-ch_d_herfwhite) + ch_d_pop_fbl^2 + ch_d_pop_fin^2 + ch_d_pop_fas^2 +
                        ch_d_pop_fhi^(3-ch_d_herfhisp));
    *JR:  Include other hispanic as a separate category.  Canadians and french canadians are "other",
     *and subsaharan african, USA, west indian, "Race or hispanic origin groups", and unclassified are excluded.;
     egen jr_d_anch_samer=rsum(anch_colo anch_ecua anch_peru anch_otsa);
     egen jr_d_anch_camer=rsum(anch_domn anch_guat anch_hond anch_nica anch_pana anch_salv anch_otca);
     egen jr_d_toteth_h=rsum(anch_mex anch_pr anch_cubn jr_d_anch_samer jr_d_anch_camer anch_oth);
     gen jr_d_anchf_mex  =   anch_mex    /jr_d_toteth_h;
     gen jr_d_anchf_pr   =   anch_pr     /jr_d_toteth_h;
     gen jr_d_anchf_cubn =   anch_cubn   /jr_d_toteth_h;
     gen jr_d_anchf_samer=jr_d_anch_samer/jr_d_toteth_h;
     gen jr_d_anchf_camer=jr_d_anch_camer/jr_d_toteth_h;
     gen jr_d_anchf_oth  =   anch_oth    /jr_d_toteth_h;
     foreach g in mex pr cubn samer camer oth {;
      gen jr_d_anchf2_`g'=jr_d_anchf_`g'^2;
      };
     egen jr_d_herfhisp=rsum(jr_d_anchf2_mex jr_d_anchf2_pr jr_d_anchf2_cubn jr_d_anchf2_samer jr_d_anchf2_camer jr_d_anchf2_oth);
     egen jr_d_anc_brit=rsum(anc_engl anc_sceir anc_scott anc_welsh);
     egen jr_d_anc_scand=rsum(anc_dansh anc_finn anc_norw anc_swed);
     egen jr_d_anc_russ=rsum(anc_russ anc_ukr);
     egen jr_d_anc_eeur=rsum(anc_czech anc_lith anc_roman anc_slov anc_yugo);
     egen jr_d_anc_benelux=rsum(anc_belg anc_dutch);
     egen jr_d_anc_swisaus=rsum(anc_swiss anc_aust);
     egen jr_d_anc_otwhite=rsum(anc_canad anc_frcan anc_otr);
     egen jr_d_totwh=rsum(jr_d_anc_brit jr_d_anc_scand jr_d_anc_russ jr_d_anc_eeur jr_d_anc_benelux jr_d_anc_swisaus
                          anc_frnch jr_d_anc_otwhite anc_arab anc_germ anc_grk anc_hung
                          anc_eir anc_ital anc_pole anc_port);
     gen jr_d_herfwhite=0;
     foreach g in brit scand russ eeur benelux swisaus otwhite {;
      replace jr_d_herfwhite=jr_d_herfwhite+(jr_d_anc_`g'/jr_d_totwh)^2 if jr_d_anc_`g'<.;
      };
     foreach g in frnch arab germ grk hung eir ital pole port {;
      replace jr_d_herfwhite=jr_d_herfwhite+(anc_`g'/jr_d_totwh)^2 if anc_`g'<.;
      };
     gen jr_d_pop_fwh3=jr_d_pop_fwh^(3-jr_d_herfwhite);
     replace jr_d_pop_fwh3=0 if jr_d_pop_fwh==.;
     gen jr_d_pop_fhi3=jr_d_pop_fhi^(3-jr_d_herfhisp);
     replace jr_d_pop_fhi3=0 if jr_d_pop_fhi==.;
     gen jr_d_herfethn=(jr_d_pop_fwh3 + jr_d_pop_fbl2 + jr_d_pop_fin2 + jr_d_pop_fas2 + jr_d_pop_fhi3 + jr_d_pop_fot2);
  recode d_stfips ( 9 23 25 33 44 50          =1)
                  (36 34 42                   =2)
                  (17 18 26 39 55             =3)
                  (19 20 27 29 31 38 46       =4)
                  (10 11 12 13 24 37 45 51 54 =5)
                  ( 1 21 28 47                =6)
                  ( 5 22 40 48                =7)
                  ( 4  8 16 30 32 35 49 56    =8)
                  ( 2  6 15 41 53             =9),
          gen(ch_d_division);
   quietly tab ch_d_division, gen(ch_d_div);
  keep d_stfips dist leaid d_pop d_numhh
       d_ed_lths d_ed_hs d_ed_scol d_ed_ba d_herfed d_lnmeanincA
       ch_d_pop_fwh ch_d_pop_fbl ch_d_pop_fhi ch_d_pop_fas ch_d_pop_fin ch_d_herfrace
       ch_d_herfhisp ch_d_herfwhite ch_d_herfethn
       ch_d_lnmeanincB ch_d_gini
       jr_d_pop_fwh jr_d_pop_fbl jr_d_pop_fin jr_d_pop_fas jr_d_pop_fhi jr_d_pop_fot jr_d_herfrace
       jr_d_herfhisp jr_d_herfwhite jr_d_herfethn
       jr_d_lnmeanincB jr_d_meanlninc jr_d_gini
       ch_d_div?
       d_lnmeanincA2 d_lnmeanincA3;
  label var dist          "SDDB district code (txt)";
  label var d_stfips      "SDDB state code";
  label var leaid         "NCES district ID";
  label var d_pop         "Dist: Total population";
  label var d_numhh       "Dist: Total # of HH";
  label var d_ed_lths       "Dist: fr. of adult pop., LTHS";
  label var d_ed_hs         "Dist: fr. of adult pop., HS";
  label var d_ed_scol       "Dist: fr. of adult pop., some college";
  label var d_ed_ba         "Dist: fr. of adult pop., BA+";
  label var d_herfed        "Dist: Educational herfindahl";
  label var d_lnmeanincA    "Dist: log(mean HH inc), from agg inc";
  label var ch_d_pop_fwh    "Dist pop: Fr. white (CH rule)";
  label var ch_d_pop_fbl    "Dist pop: Fr. black (CH rule)";
  label var ch_d_pop_fhi    "Dist pop: Fr. hisp (CH rule)";
  label var ch_d_pop_fas    "Dist pop: Fr. asian (CH rule)";
  label var ch_d_pop_fin    "Dist pop: Fr. indian (CH rule)";
  label var ch_d_herfrace   "Dist: Racial herfindahl (CH rule)";
  label var ch_d_herfhisp   "Dist: Ethnic herf. among hispanics (CH rule)";
  label var ch_d_herfwhite  "Dist: Ethnic herf. among whites (CH rule)";
  label var ch_d_herfethn   "Dist: Ethnic-adj. racial herf. (CH rule)";
  label var ch_d_lnmeanincB "Dist: log(mean HH inc), from bins (CH rule)";
  label var ch_d_gini       "Dist: Gini coefficient (CH rule)";
  label var jr_d_pop_fwh    "Dist pop: Fr. white (JR rule)";
  label var jr_d_pop_fbl    "Dist pop: Fr. black (JR rule)";
  label var jr_d_pop_fhi    "Dist pop: Fr. hisp (JR rule)";
  label var jr_d_pop_fas    "Dist pop: Fr. asian (JR rule)";
  label var jr_d_pop_fin    "Dist pop: Fr. indian (JR rule)";
  label var jr_d_pop_fot    "Dist pop: Fr. other race (JR rule)";
  label var jr_d_herfrace   "Dist: Racial herfindahl (JR rule)";
  label var jr_d_herfhisp   "Dist: Ethnic herf. among hispanics (JR rule)";
  label var jr_d_herfwhite  "Dist: Ethnic herf. among whites (JR rule)";
  label var jr_d_herfethn   "Dist: Ethnic-adj. racial herf. (JR rule)";
  label var jr_d_lnmeanincB "Dist: log(mean HH inc), from bins (JR rule)";
  label var jr_d_meanlninc  "Dist: mean(log HH inc), from bins";
  label var jr_d_gini       "Dist: Gini coefficient (JR rule)";
  label var ch_d_div1     "Dist: in division 1";
  label var ch_d_div2     "Dist: in division 2";
  label var ch_d_div3     "Dist: in division 3";
  label var ch_d_div4     "Dist: in division 4";
  label var ch_d_div5     "Dist: in division 5";
  label var ch_d_div6     "Dist: in division 6";
  label var ch_d_div7     "Dist: in division 7";
  label var ch_d_div8     "Dist: in division 8";
  label var ch_d_div9     "Dist: in division 9";
  tempfile sddbdata;
  sort leaid;
  duplicates report leaid;
  compress;
  save `sddbdata';

 *Read Hoxby top-100 file for per-pupil spending, etc.;
  clear;
  insheet using ${constructdir_public}/sddb/sddb.top100.csv, comma;
  keep st dist students teachers totexp rc;
  gen pps=totexp*1000/students;
  *Following Hoxby, censor at 1% and 99%;
  xtile centilepps=pps, nquantiles(100);
  gen lnpps=log(pps);
  gen lnpps_cens=lnpps if centilepps>1 & centilepps<100;
  label var pps "total dist expend/student, uncensored";
  label var lnpps "ln(total dist expend/student), uncensored";
  label var lnpps_cens "ln(total dist expend/student), censored";
  gen stratio=students/teachers;
  xtile centilest=stratio, nquantiles(100);
  gen stratio_cens=stratio if centilest>1 & centilest<100;
  gen tsratio=teachers/students;
  xtile centilets=tsratio, nquantiles(100);
  gen tsratio_cens=tsratio if centilets>1 & centilets<100;
  label var stratio "student/teacher ratio, uncensored";
  label var stratio_cens "student/teacher ratio, censored";
  label var tsratio "teacher/student ratio, uncensored";
  label var tsratio_cens "teacher/student ratio, censored";
  drop if dist==0;
  gen leaid=st*100000+dist;
  keep leaid lnpps lnpps_cens pps stratio stratio_cens tsratio tsratio_cens students rc;
  sort leaid;
  duplicates report leaid;
  compress;
  merge leaid using `sddbdata', unique _merge(top100mrg);
  tab top100mrg;
  drop top100mrg;
  save `sddbdata', replace;

 *Merge SDDB to CCD-MSA crosswalk;
  use `ccd2msa';
  gen leaid_num=real(leaid);
  drop leaid;
  rename leaid_num leaid;
  sort leaid;
  drop if leaid==. | leaid==0;
  keep leaid fipsmsa ccd_macode;
  tempfile ccd2msa_num;
  save `ccd2msa_num';
  use `sddbdata';
  sort leaid;
  merge leaid using `ccd2msa_num', unique _merge(lea_sddb2ccd);
  tab lea_sddb2ccd;
  *Many of the SDDB-and-not-CCD districts are in Hawaii, where CCD has one dist and SDDB has many subdists;
  *Similar situation in NYC;
   drop if lea_sddb2ccd~=3;
   drop lea_sddb2ccd;

  save ${mywork_public}/distdata.dta, replace;
  d;
  su;

 *Finally, compute an MSA-level income variable analogous to Hoxbys:  Avg. of districts log mean incomes;
  collapse (mean) d_lnmeanincA [fw=d_numhh], by(fipsmsa);
  drop if fipsmsa==.;
  rename d_lnmeanincA ch_m_avglnmeaninc;
  label var ch_m_avglnmeaninc "Avg. across districts of log(mean HH inc)";
  sort fipsmsa;
  merge fipsmsa using ${mywork_public}/msadata.dta, unique _merge(lastmrg);
  tab lastmrg;
  drop lastmrg;
  save ${mywork_public}/msadata.dta, replace;

log close;